@@ -33,6 +33,9 @@ from ..model.DioptasModel import DioptasModel
 
 
 class MaskController(object):
+    DEFAULT_MASK_FILTER = 'Mask (*.mask)'
+    FLIPUD_MASK_FILTER_PREFIX = 'Vertically flipped mask'
+
     def __init__(self, widget, dioptas_model):
         """
         :param widget: Reference to a MaskView object
@@ -55,10 +58,6 @@ class MaskController(object):
 
     def create_signals(self):
         self.widget.img_widget.mouse_left_clicked.connect(self.process_click)
-
-        self.model.img_changed.connect(self.update_mask_dimension)
-        self.model.configuration_selected.connect(self.update_gui)
-
         self.widget.circle_btn.clicked.connect(self.activate_circle_btn)
         self.widget.rectangle_btn.clicked.connect(self.activate_rectangle_btn)
         self.widget.polygon_btn.clicked.connect(self.activate_polygon_btn)
@@ -86,6 +85,25 @@ class MaskController(object):
 
         self.widget.keyPressEvent = self.key_press_event
 
+        self.model.img_changed.connect(self.update_mask_dimension)
+        self.model.configuration_selected.connect(self.update_gui)
+
+    def activate_model_signals(self):
+        if not self.model.img_changed.has_listener(self.update_mask_dimension):
+            self.model.img_changed.connect(self.update_mask_dimension)
+        if not self.model.configuration_selected.has_listener(self.update_gui):
+            self.model.configuration_selected.connect(self.update_gui)
+
+    def activate(self):
+        self.activate_model_signals()
+        self.update_gui()
+
+    def deactivate(self):
+        if self.model.img_changed.has_listener(self.update_mask_dimension):
+            self.model.img_changed.disconnect(self.update_mask_dimension)
+        if self.model.configuration_selected.has_listener(self.update_gui):
+            self.model.configuration_selected.disconnect(self.update_gui)
+
     def update_mask_dimension(self):
         self.model.mask_model.set_dimension(self.model.img_model._img_data.shape)
 
@@ -264,7 +282,7 @@ class MaskController(object):
             self.widget.img_widget.mouse_moved.disconnect(self.arc_calc_and_preview)
             if (self.arc.vertices[0].x() == self.arc.vertices[2].x() and self.arc.vertices[0].y() == self.arc.vertices[
                 2].y()) or (self.arc.vertices[1].x() == self.arc.vertices[2].x() and self.arc.vertices[1].y() ==
-                self.arc.vertices[2].y()):
+                            self.arc.vertices[2].y()):
                 self.remove_bad_arc()
                 return
             self.widget.img_widget.mouse_moved.connect(self.arc_width_preview)
@@ -370,22 +388,36 @@ class MaskController(object):
 
     def save_mask_btn_click(self):
         img_filename, _ = os.path.splitext(os.path.basename(self.model.img_model.filename))
-        filename = save_file_dialog(self.widget, "Save mask data",
-                                    os.path.join(self.model.working_directories['mask'],
-                                                 img_filename + '.mask'),
-                                    filter='Mask (*.mask)')
+        filename, selected_filter = QtWidgets.QFileDialog.getSaveFileName(
+            self.widget,
+            caption="Save mask data",
+            directory=os.path.join(self.model.working_directories['mask'],
+                                   img_filename + '.mask'),
+            filter=';;'.join([
+                self.DEFAULT_MASK_FILTER,
+                f"{self.FLIPUD_MASK_FILTER_PREFIX} (*.npy)",
+                f"{self.FLIPUD_MASK_FILTER_PREFIX} (*.edf)",
+            ]))
 
         if filename != '':
+            flipud = selected_filter.startswith(self.FLIPUD_MASK_FILTER_PREFIX)
             self.model.working_directories['mask'] = os.path.dirname(filename)
-            self.model.mask_model.save_mask(filename)
+            self.model.mask_model.save_mask(filename, flipud)
 
     def load_mask_btn_click(self):
-        filename = open_file_dialog(self.widget, caption="Load mask data",
-                                    directory=self.model.working_directories['mask'], filter='*.mask')
+        filename, selected_filter = QtWidgets.QFileDialog.getOpenFileName(
+            self.widget,
+            caption="Load mask data",
+            directory=self.model.working_directories['mask'],
+            filter=';;'.join([
+                self.DEFAULT_MASK_FILTER,
+                f"{self.FLIPUD_MASK_FILTER_PREFIX} (*.npy *.edf)",
+            ]))
 
         if filename != '':
+            flipud = selected_filter.startswith(self.FLIPUD_MASK_FILTER_PREFIX)
             self.model.working_directories['mask'] = os.path.dirname(filename)
-            if self.model.mask_model.load_mask(filename):
+            if self.model.mask_model.load_mask(filename, flipud):
                 self.plot_mask()
             else:
                 QtWidgets.QMessageBox.critical(self.widget, 'Error',
@@ -393,12 +425,19 @@ class MaskController(object):
                                                'the same shape. Mask could not be loaded.')
 
     def add_mask_btn_click(self):
-        filename = open_file_dialog(self.widget, caption="Add mask data",
-                                    directory=self.model.working_directories['mask'], filter='*.mask')
+        filename, selected_filter = QtWidgets.QFileDialog.getOpenFileName(
+            self.widget,
+            caption="Add mask data",
+            directory=self.model.working_directories['mask'],
+            filter=';;'.join([
+                self.DEFAULT_MASK_FILTER,
+                f"{self.FLIPUD_MASK_FILTER_PREFIX} (*.npy *.edf)",
+            ]))
 
         if filename != '':
+            flipud = selected_filter.startswith(self.FLIPUD_MASK_FILTER_PREFIX)
             self.model.working_directories['mask'] = os.path.dirname(filename)
-            if self.model.mask_model.add_mask(filename):
+            if self.model.mask_model.add_mask(filename, flipud):
                 self.plot_mask()
             else:
                 QtWidgets.QMessageBox.critical(self.widget, 'Error',
@@ -460,7 +499,7 @@ class MaskController(object):
         self.widget.pos_lbl.setText(str)
 
     def update_gui(self):
-        #transparency
+        # transparency
         if self.model.transparent_mask:
             self.widget.transparent_rb.setChecked(True)
             self.transparent_rb_click()
@@ -470,10 +509,3 @@ class MaskController(object):
 
         self.plot_mask()
         self.plot_image()
-
-
-
-if __name__ == "__main__":
-    app = QtWidgets.QApplication(sys.argv)
-    controller = MaskController()
-    app.exec_()
