@@ -19,17 +19,17 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-from functools import partial
 
 import numpy as np
 from PIL import Image
 from qtpy import QtWidgets, QtCore
+import pyqtgraph as pg
+from xypattern import Pattern
 
 from ...widgets.UtilityWidgets import open_file_dialog, open_files_dialog, save_file_dialog
 # imports for type hinting in PyCharm -- DO NOT DELETE
 from ...widgets.integration import IntegrationWidget
 from ...model.DioptasModel import DioptasModel
-from ...model.util.Pattern import Pattern
 from ...model.util.HelperModule import get_partial_index, get_partial_value
 
 from .EpicsController import EpicsController
@@ -41,13 +41,10 @@ class ImageController(object):
     well as interaction with the image_view.
     """
 
-    def __init__(self, widget, dioptas_model):
+    def __init__(self, widget: IntegrationWidget, dioptas_model: DioptasModel):
         """
-        :param widget: Reference to IntegrationView
+        :param widget: Reference to IntegrationWidget
         :param dioptas_model: Reference to DioptasModel object
-
-        :type widget: IntegrationWidget
-        :type dioptas_model: DioptasModel
         """
         self.widget = widget
         self.model = dioptas_model
@@ -58,29 +55,19 @@ class ImageController(object):
         self.view_mode = 'normal'  # modes available: normal, alternative
         self.roi_active = False
 
-        self.clicked_tth = None
-        self.clicked_azi = None
-
         self.vertical_splitter_alternative_state = None
         self.vertical_splitter_normal_state = None
         self.horizontal_splitter_alternative_state = None
         self.horizontal_splitter_normal_state = None
 
-        self.initialize()
         self.create_signals()
         self.create_mouse_behavior()
 
-    def initialize(self):
-        self.update_img()
-        self.plot_img()
-        self.plot_mask()
-        self.widget.img_widget.auto_level()
-
     def plot_img(self, auto_scale=None):
         """
         Plots the current image loaded in self.img_data.
         :param auto_scale:
-            Determines if intensities shouldk be auto-scaled. If value is None it will use the parameter saved in the
+            Determines if intensities should be auto-scaled. If value is None it will use the parameter saved in the
             Object (self._auto_scale)
         """
         if auto_scale is None:
@@ -112,11 +99,11 @@ class ImageController(object):
             self.widget.cake_widget.auto_level()
 
     def plot_cake_integral(self, tth=None):
-        if not self.widget.cake_widget.cake_integral_plot.isVisible() or self.clicked_tth is None:
+        if not self.widget.cake_widget.cake_integral_plot.isVisible():
             return
 
         if tth is None:
-            tth = self.clicked_tth
+            tth = self.model.clicked_tth
 
         x, y = self.model.calibration_model.cake_integral(
             tth,
@@ -125,6 +112,9 @@ class ImageController(object):
         shift_amount = self.widget.cake_shift_azimuth_sl.value()
         self.widget.cake_widget.plot_cake_integral(x, np.roll(y, shift_amount))
 
+    def _update_cake_integral(self):
+        self.plot_cake_integral()
+
     def save_cake_integral(self):
         img_filename, _ = os.path.splitext(os.path.basename(self.model.img_model.filename))
         filename = save_file_dialog(
@@ -141,11 +131,10 @@ class ImageController(object):
         Plots the mask data.
         """
         if self.model.use_mask and self.widget.img_mode == 'Image':
+            self.widget.img_widget.activate_mask()
             self.widget.img_widget.plot_mask(self.model.mask_model.get_img())
-            self.widget.img_mask_btn.setChecked(True)
         else:
-            self.widget.img_widget.plot_mask(np.zeros(self.model.mask_model.get_img().shape))
-            self.widget.img_mask_btn.setChecked(False)
+            self.widget.img_widget.deactivate_mask()
 
     def update_mask_transparency(self):
         """
@@ -160,7 +149,7 @@ class ImageController(object):
 
     def create_signals(self):
         self.model.configuration_selected.connect(self.update_gui_from_configuration)
-        self.model.img_changed.connect(self.update_img)
+        self.model.img_changed.connect(self.update_img_control_widget)
 
         self.model.img_changed.connect(self.plot_img)
         self.model.img_changed.connect(self.plot_mask)
@@ -183,25 +172,26 @@ class ImageController(object):
 
         self.widget.img_step_file_widget.browse_by_name_rb.clicked.connect(self.set_iteration_mode_number)
         self.widget.img_step_file_widget.browse_by_time_rb.clicked.connect(self.set_iteration_mode_time)
-        self.widget.mask_transparent_cb.clicked.connect(self.update_mask_transparency)
+
+        self.widget.image_control_widget.sources_cb.currentTextChanged.connect(self.select_source)
 
         ###
         # Image widget image specific controls
         self.widget.img_roi_btn.clicked.connect(self.click_roi_btn)
         self.widget.img_mask_btn.clicked.connect(self.change_mask_mode)
+        self.widget.mask_transparent_cb.clicked.connect(self.update_mask_transparency)
 
         ###
         # Image Widget cake specific controls
         self.widget.img_phases_btn.clicked.connect(self.toggle_show_phases)
-        self.widget.cake_shift_azimuth_sl.valueChanged.connect(partial(self.plot_cake, None))
-        self.widget.cake_shift_azimuth_sl.valueChanged.connect(self._update_cake_mouse_click_pos)
-        self.widget.cake_shift_azimuth_sl.valueChanged.connect(self.update_cake_azimuth_axis)
-        self.widget.cake_shift_azimuth_sl.valueChanged.connect(partial(self.plot_cake_integral, None))
+        self.widget.cake_shift_azimuth_sl.valueChanged.connect(self.cake_shift_changed)
         self.widget.integration_image_widget.cake_view.img_view_box.sigRangeChanged.connect(self.update_cake_axes_range)
-        self.widget.pattern_q_btn.clicked.connect(partial(self.set_cake_axis_unit, 'q_A^-1'))
-        self.widget.pattern_tth_btn.clicked.connect(partial(self.set_cake_axis_unit, '2th_deg'))
+        self.widget.pattern_q_btn.clicked.connect(self.set_cake_axis_to_q)
+        self.widget.pattern_tth_btn.clicked.connect(self.set_cake_axis_to_2th)
         self.widget.integration_control_widget.integration_options_widget.cake_integral_width_sb.valueChanged. \
-            connect(partial(self.plot_cake_integral, None))
+            connect(self._update_cake_integral)
+        self.widget.integration_control_widget.integration_options_widget.cake_integral_width_sb.editingFinished. \
+            connect(self._update_cake_integral)
         self.widget.integration_control_widget.integration_options_widget.cake_save_integral_btn.clicked. \
             connect(self.save_cake_integral)
 
@@ -216,11 +206,43 @@ class ImageController(object):
 
         self.widget.qa_save_img_btn.clicked.connect(self.save_img)
         self.widget.load_calibration_btn.clicked.connect(self.load_calibration)
+        self.widget.set_wavelnegth_btn.clicked.connect(self.set_wavelength)
 
         # signals
         self.widget.change_view_btn.clicked.connect(self.change_view_btn_clicked)
         self.widget.autoprocess_cb.toggled.connect(self.auto_process_cb_click)
 
+    def activate(self):
+        if self.widget.img_mode == 'Image':
+            if not self.model.img_changed.has_listener(self.plot_img):
+                self.model.img_changed.connect(self.plot_img)
+            if not self.model.img_changed.has_listener(self.plot_mask):
+                self.model.img_changed.connect(self.plot_mask)
+        elif self.widget.img_mode == 'Cake':
+            if not self.model.cake_changed.has_listener(self.plot_cake):
+                self.model.cake_changed.connect(self.plot_cake)
+        self.widget.calibration_lbl.setText(
+            self.model.calibration_model.calibration_name
+        )
+        self.widget.wavelength_lbl.setText(
+            "{:.4f}".format(self.model.calibration_model.wavelength * 1e10) + " A"
+        )
+
+    def update_image(self):
+        if self.widget.img_mode == 'Image':
+            self.plot_mask()
+            self.plot_img()
+        elif self.widget.img_mode == 'Cake':
+            self.plot_cake()
+
+    def deactivate(self):
+        if self.model.img_changed.has_listener(self.plot_img):
+            self.model.img_changed.disconnect(self.plot_img)
+        if self.plot_mask in self.model.img_changed.listeners:
+            self.model.img_changed.disconnect(self.plot_mask)
+        if self.plot_cake in self.model.cake_changed.listeners:
+            self.model.cake_changed.disconnect(self.plot_cake)
+
     def create_mouse_behavior(self):
         """
         Creates the signal connections of mouse interactions
@@ -230,7 +252,8 @@ class ImageController(object):
         self.widget.cake_widget.mouse_left_clicked.connect(self.img_mouse_click)
         self.widget.cake_widget.mouse_moved.connect(self.show_img_mouse_position)
 
-        self.widget.pattern_widget.mouse_left_clicked.connect(self.pattern_mouse_click)
+        self.model.clicked_tth_changed.connect(self.set_cake_line_position)
+        self.model.clicked_tth_changed.connect(self.set_image_line_position)
 
     def load_file(self, *args, **kwargs):
         filename = kwargs.get('filename', None)
@@ -252,6 +275,15 @@ class ImageController(object):
                         self.model.img_model.add(filenames[ind])
                     self.model.img_model.blockSignals(False)
                     self.model.img_model.img_changed.emit()
+                if self.widget.img_batch_mode_average_rb.isChecked():
+                    self.model.img_model.blockSignals(True)
+                    self.model.img_model.load(str(filenames[0]))
+                    for ind in range(1, len(filenames)):
+                        self.model.img_model.add(filenames[ind])
+                    self.model.img_model._img_data = self.model.img_model._img_data / len(filenames)
+                    self.model.img_model._calculate_img_data()
+                    self.model.img_model.blockSignals(False)
+                    self.model.img_model.img_changed.emit()
                 elif self.widget.img_batch_mode_integrate_rb.isChecked():
                     self._load_multiple_files(filenames)
                 elif self.widget.img_batch_mode_image_save_rb.isChecked():
@@ -353,7 +385,7 @@ class ImageController(object):
                 self.model.pattern_model.save_pattern(filename)
 
             # save the background subtracted filename
-            if self.model.pattern.has_background():
+            if self.model.pattern.auto_bkg is not None:
                 directory = os.path.join(working_directory, 'bkg_subtracted')
                 if not os.path.exists(directory):
                     os.mkdir(directory)
@@ -435,9 +467,9 @@ class ImageController(object):
         self.model.img_model.img_changed.emit()
 
     def update_mask_mode(self):
-        self.widget.integration_image_widget.mask_btn.setChecked(self.model.use_mask)
-        self.widget.mask_transparent_cb.setVisible(self.model.use_mask)
-        self.widget.mask_transparent_cb.setChecked(self.model.transparent_mask)
+        self.widget.integration_image_widget.mask_btn.setChecked(bool(self.model.use_mask))
+        self.widget.mask_transparent_cb.setVisible(bool(self.model.use_mask))
+        self.widget.mask_transparent_cb.setChecked(bool(self.model.transparent_mask))
 
     def update_img_mode(self):
         self.widget.img_mode_btn.click()
@@ -470,7 +502,7 @@ class ImageController(object):
         new_filename = str(self.widget.img_filename_txt.text())
         if os.path.exists(os.path.join(current_directory, new_filename)):
             try:
-                self.load_file(filename=os.path.join(current_directory, new_filename))
+                self.load_file(filename=os.path.join(current_directory, new_filename).replace('\\', '/'))
             except TypeError:
                 self.widget.img_filename_txt.setText(current_filename)
         else:
@@ -498,8 +530,8 @@ class ImageController(object):
             self.model.working_directories['image'] = directory
             self.widget.img_directory_txt.setText(directory)
 
-    def update_img(self):
-        self.widget.img_step_series_widget.setVisible(self.model.img_model.series_max > 1)
+    def update_img_control_widget(self):
+        self.widget.img_step_series_widget.setVisible(int(self.model.img_model.series_max > 1))
         self.widget.img_step_series_widget.pos_validator.setTop(self.model.img_model.series_max)
         self.widget.img_step_series_widget.pos_txt.setText(str(self.model.img_model.series_pos))
 
@@ -510,6 +542,18 @@ class ImageController(object):
         self.widget.img_directory_txt.setText(os.path.dirname(self.model.img_model.filename))
         self.widget.file_info_widget.text_lbl.setText(self.model.img_model.file_info)
 
+        self.widget.image_control_widget.sources_widget.setVisible(not (self.model.img_model.sources is None))
+        if self.model.img_model.sources is not None:
+            sources_cb = self.widget.image_control_widget.sources_cb
+            sources_cb.blockSignals(True)
+            # remove all previous items:
+            for _ in range(sources_cb.count()):
+                sources_cb.removeItem(0)
+
+            sources_cb.addItems(self.model.img_model.sources)
+            sources_cb.setCurrentText(self.model.img_model.selected_source)
+            sources_cb.blockSignals(False)
+
         self.widget.cbn_plot_btn.setText('Plot')
         self.widget.oiadac_plot_btn.setText('Plot')
 
@@ -544,8 +588,8 @@ class ImageController(object):
             self.widget.img_widget.activate_roi()
             self.widget.img_widget.update_roi_shade_limits(self.model.img_data.shape)
 
-            pos = QtCore.QPoint(roi[2], roi[0])
-            size = QtCore.QPoint(roi[3] - roi[2], roi[1] - roi[0])
+            pos = QtCore.QPoint(int(roi[2]), int(roi[0]))
+            size = QtCore.QPoint(int(roi[3] - roi[2]), int(roi[1] - roi[0]))
             self.widget.img_widget.roi.setRoiLimits(pos, size)
 
             if not self.roi_active:
@@ -571,13 +615,23 @@ class ImageController(object):
             self.widget.integration_image_widget.cake_view.hide_all_cake_phases()
             self.widget.img_phases_btn.setText('Show Phases')
 
+    def cake_shift_changed(self):
+        self.plot_cake()
+        self._update_cake_mouse_click_pos()
+        self.update_cake_azimuth_axis()
+        self.plot_cake_integral(None)
+
     def activate_cake_mode(self):
+        if self.model.calibration_model.cake_geometry is None:
+            self.widget.show_error_msg("Can not switch to the cake mode without calibration.")
+            return
+
         if not self.model.current_configuration.auto_integrate_cake:
             self.model.current_configuration.auto_integrate_cake = True
 
         self.model.current_configuration.integrate_image_2d()
 
-        self._update_cake_line_pos()
+        self.set_cake_line_position(self.model.clicked_tth)
         self._update_cake_mouse_click_pos()
 
         self.widget.img_mode_btn.setText('Image')
@@ -590,8 +644,8 @@ class ImageController(object):
         self.plot_cake()
 
         self.widget.cake_shift_azimuth_sl.setVisible(True)
-        self.widget.cake_shift_azimuth_sl.setMinimum(-len(self.model.cake_azi) / 2)
-        self.widget.cake_shift_azimuth_sl.setMaximum(len(self.model.cake_azi) / 2)
+        self.widget.cake_shift_azimuth_sl.setMinimum(int(-len(self.model.cake_azi) / 2))
+        self.widget.cake_shift_azimuth_sl.setMaximum(int(len(self.model.cake_azi) / 2))
         self.widget.cake_shift_azimuth_sl.setSingleStep(1)
         self.widget.img_phases_btn.setVisible(True)
 
@@ -633,26 +687,20 @@ class ImageController(object):
         else:
             self.widget.integration_image_widget.show_background_subtracted_img_btn.setChecked(False)
 
-    def _update_cake_line_pos(self):
-        cur_tth = self.get_current_pattern_tth()
-        if self.model.cake_tth is None or cur_tth < np.min(self.model.cake_tth) or cur_tth > np.max(
-                self.model.cake_tth):
-            self.widget.cake_widget.deactivate_vertical_line()
-        else:
-            new_pos = get_partial_index(self.model.cake_tth, cur_tth) + 0.5
-            self.widget.cake_widget.set_vertical_line_pos(new_pos, 0)
-            self.widget.cake_widget.activate_vertical_line()
-
     def _update_cake_mouse_click_pos(self):
-        if self.clicked_tth is None or not self.model.calibration_model.is_calibrated:
+        if not self.model.calibration_model.is_calibrated:
             return
 
-        tth = self.clicked_tth
-        azi = self.clicked_azi
+        tth = self.model.clicked_tth
+        azi = self.model.clicked_azi
 
         cake_tth = self.model.cake_tth
 
-        x_pos = get_partial_index(cake_tth, tth) + 0.5
+        x_pos = get_partial_index(cake_tth, tth)
+        if x_pos is None:
+            return
+
+        x_pos = x_pos + 0.5
         shift_amount = self.widget.cake_shift_azimuth_sl.value()
         y_pos = (get_partial_index(self.model.cake_azi, azi) + 0.5 + shift_amount) % len(self.model.cake_azi)
         self.widget.cake_widget.set_mouse_click_position(x_pos, y_pos)
@@ -660,16 +708,14 @@ class ImageController(object):
     def _update_image_line_pos(self):
         if not self.model.calibration_model.is_calibrated:
             return
-        cur_tth = self.get_current_pattern_tth()
-        self.widget.img_widget.set_circle_line(
-            self.model.calibration_model.get_two_theta_array(), cur_tth / 180 * np.pi)
+        self.set_image_line_position(self.model.clicked_tth)
 
     def _update_image_mouse_click_pos(self):
-        if self.clicked_tth is None or not self.model.calibration_model.is_calibrated:
+        if not self.model.calibration_model.is_calibrated:
             return
 
-        tth = np.deg2rad(self.clicked_tth)
-        azi = np.deg2rad(self.clicked_azi)
+        tth = np.deg2rad(self.model.clicked_tth)
+        azi = np.deg2rad(self.model.clicked_azi)
 
         new_pos = self.model.calibration_model.get_pixel_ind(tth, azi)
         if len(new_pos) == 0:
@@ -679,31 +725,22 @@ class ImageController(object):
             self.widget.img_widget.set_mouse_click_position(y_ind + 0.5, x_ind + 0.5)
             self.widget.img_widget.mouse_click_item.show()
 
-    def get_current_pattern_tth(self):
-        cur_pos = self.widget.pattern_widget.pos_line.getPos()[0]
-        if self.widget.pattern_q_btn.isChecked():
-            cur_tth = self.convert_x_value(cur_pos, 'q_A^-1', '2th_deg')
-        elif self.widget.pattern_tth_btn.isChecked():
-            cur_tth = cur_pos
-        elif self.widget.pattern_d_btn.isChecked():
-            cur_tth = self.convert_x_value(cur_pos, 'd_A', '2th_deg')
-        else:
-            cur_tth = None
-        return cur_tth
-
     def update_cake_axes_range(self):
         if self.model.current_configuration.auto_integrate_cake:
             self.update_cake_azimuth_axis()
             self.update_cake_x_axis()
 
     def update_cake_azimuth_axis(self):
-        data_img_item = self.widget.integration_image_widget.cake_view.data_img_item
+        img_view_rect = self.widget.integration_image_widget.cake_view.img_view_rect()
+        img_bounding_rect = self.widget.integration_image_widget.cake_view.img_bounding_rect()
         shift_amount = self.widget.cake_shift_azimuth_sl.value()
         cake_azi = self.model.cake_azi - shift_amount * np.mean(np.diff(self.model.cake_azi))
+        if img_bounding_rect.height() == 0:
+            return
 
-        height = data_img_item.viewRect().height()
-        bottom = data_img_item.viewRect().top()
-        v_scale = (cake_azi[-1] - cake_azi[0]) / data_img_item.boundingRect().height()
+        height = img_view_rect.height()
+        bottom = img_view_rect.top()
+        v_scale = (cake_azi[-1] - cake_azi[0]) / img_bounding_rect.height()
         v_shift = np.min(cake_azi[0])
         min_azi = v_scale * bottom + v_shift
         max_azi = v_scale * (bottom + height) + v_shift
@@ -714,12 +751,16 @@ class ImageController(object):
         if self.model.cake_tth is None:
             return
 
-        data_img_item = self.widget.integration_image_widget.cake_view.data_img_item
-        cake_tth = self.model.cake_tth
+        img_view_rect = self.widget.integration_image_widget.cake_view.img_view_rect()
+        img_bounding_rect = self.widget.integration_image_widget.cake_view.img_bounding_rect()
 
-        width = data_img_item.viewRect().width()
-        left = data_img_item.viewRect().left()
-        h_scale = (np.max(cake_tth) - np.min(cake_tth)) / data_img_item.boundingRect().width()
+        cake_tth = self.model.cake_tth
+        if img_bounding_rect.width() == 0:
+            return
+
+        width = img_view_rect.width()
+        left = img_view_rect.left()
+        h_scale = (np.max(cake_tth) - np.min(cake_tth)) / img_bounding_rect.width()
         h_shift = np.min(cake_tth)
         min_tth = h_scale * left + h_shift
         max_tth = h_scale * (left + width) + h_shift
@@ -731,11 +772,12 @@ class ImageController(object):
                 self.convert_x_value(min_tth, '2th_deg', 'q_A^-1'),
                 self.convert_x_value(max_tth, '2th_deg', 'q_A^-1'))
 
-    def set_cake_axis_unit(self, unit='2th_deg'):
-        if unit == '2th_deg':
-            self.widget.integration_image_widget.cake_view.bottom_axis_cake.setLabel(u'2θ', u'°')
-        elif unit == 'q_A^-1':
-            self.widget.integration_image_widget.cake_view.bottom_axis_cake.setLabel('Q', 'A<sup>-1</sup>')
+    def set_cake_axis_to_2th(self):
+        self.widget.integration_image_widget.cake_view.bottom_axis_cake.setLabel(u'2θ', u'°')
+        self.update_cake_x_axis()
+
+    def set_cake_axis_to_q(self):
+        self.widget.integration_image_widget.cake_view.bottom_axis_cake.setLabel('Q', 'A<sup>-1</sup>')
         self.update_cake_x_axis()
 
     def show_img_mouse_position(self, x, y):
@@ -747,7 +789,10 @@ class ImageController(object):
 
         if 0 < x < img_shape[1] - 1 and 0 < y < img_shape[0] - 1:
             self.update_mouse_position_labels(x, y, img_data[int(np.floor(y)), int(np.floor(x))])
+        else:
+            self.update_mouse_position_labels(x, y, None)
 
+        if 0.5 < x < img_shape[1] - 0.5 and 0.5 < y < img_shape[0] - 0.5:
             if self.model.calibration_model.is_calibrated:
                 x_temp = x
                 x = np.array([y])
@@ -783,8 +828,6 @@ class ImageController(object):
                 self.widget.img_widget_mouse_d_lbl.setText('d: -')
                 self.widget.img_widget_mouse_q_lbl.setText('Q: -')
                 self.widget.img_widget_mouse_azi_lbl.setText('X: -')
-        else:
-            self.update_mouse_position_labels(x, y, None)
 
     def img_mouse_click(self, x, y):
         if self.widget.img_mode == 'Cake':
@@ -805,8 +848,6 @@ class ImageController(object):
                 cake_shape = self.model.cake_data.shape
                 if x < 0 or y < 0 or x > cake_shape[0] - 1 or y > cake_shape[1] - 1:
                     return
-                x = np.array([x])
-                y = np.array([y])
                 tth = get_partial_value(self.model.cake_tth, y - 0.5)
                 shift_amount = self.widget.cake_shift_azimuth_sl.value()
                 azi = get_partial_value(np.roll(self.model.cake_azi, shift_amount), x - 0.5)
@@ -820,8 +861,6 @@ class ImageController(object):
                 y = np.array([y])
                 tth = np.rad2deg(self.model.calibration_model.get_two_theta_img(x, y))
                 azi = np.rad2deg(self.model.calibration_model.get_azi_img(x, y))
-                self.widget.img_widget.set_circle_line(self.model.calibration_model.get_two_theta_array(),
-                                                       np.deg2rad(tth))
             else:  # in the case of whatever
                 tth = 0
                 azi = 0
@@ -832,16 +871,9 @@ class ImageController(object):
             if self.widget.img_mode == 'Cake':
                 self.plot_cake_integral()
 
-            # calculate right unit for the position line the pattern widget
-            if self.widget.pattern_q_btn.isChecked():
-                pos = 4 * np.pi * np.sin(np.deg2rad(tth) / 2) / self.model.calibration_model.wavelength / 1e10
-            elif self.widget.pattern_tth_btn.isChecked():
-                pos = tth
-            elif self.widget.pattern_d_btn.isChecked():
-                pos = self.model.calibration_model.wavelength / (2 * np.sin(np.deg2rad(tth) / 2)) * 1e10
-            else:
-                pos = 0
-            self.widget.pattern_widget.set_pos_line(pos)
+            self.model.clicked_tth_changed.emit(tth)
+            self.model.clicked_azi_changed.emit(azi)
+
         self.widget.click_tth_lbl.setText(self.widget.mouse_tth_lbl.text())
         self.widget.click_d_lbl.setText(self.widget.mouse_d_lbl.text())
         self.widget.click_q_lbl.setText(self.widget.mouse_q_lbl.text())
@@ -854,10 +886,11 @@ class ImageController(object):
     def update_mouse_position_labels(self, x, y, intensity):
         x_pos_string = 'X:  %4d' % x
         y_pos_string = 'Y:  %4d' % y
-        if intensity is None:
-            int_string = 'I:'
-        else:
+
+        try:
             int_string = 'I:   %5d' % intensity
+        except (ValueError, TypeError, OverflowError):
+            int_string = 'I:'
 
         self.widget.mouse_x_lbl.setText(x_pos_string)
         self.widget.mouse_y_lbl.setText(y_pos_string)
@@ -870,42 +903,26 @@ class ImageController(object):
         self.widget.click_y_lbl.setText(self.widget.mouse_y_lbl.text())
         self.widget.click_int_lbl.setText(self.widget.mouse_int_lbl.text())
 
-    def pattern_mouse_click(self, x, y):
-        if self.model.calibration_model.is_calibrated:
-            if self.widget.img_mode == 'Cake':
-                self.set_cake_line_position(x)
-            elif self.widget.img_mode == 'Image':
-                self.set_image_line_position(x)
+    def set_cake_line_position(self, tth):
+        if self.model.cake_tth is None:
+            return
 
-    def set_cake_line_position(self, x):
-        x = self._convert_to_tth(x)
-
-        upper_ind = np.where(self.model.cake_tth > x)[0]
-        lower_ind = np.where(self.model.cake_tth < x)[0]
-
-        if len(upper_ind) == 0 or len(lower_ind) == 0:
+        pos = get_partial_index(self.model.cake_tth, tth)
+        if pos is None:
             self.widget.cake_widget.plot_cake_integral(np.array([]), np.array([]))
             self.widget.cake_widget.deactivate_vertical_line()
             return
 
-        spacing = self.model.cake_tth[upper_ind[0]] - self.model.cake_tth[lower_ind[-1]]
-        new_pos = lower_ind[-1] + (x - self.model.cake_tth[lower_ind[-1]]) / spacing + 0.5
-        self.widget.cake_widget.vertical_line.setValue(new_pos)
+        self.widget.cake_widget.vertical_line.setValue(pos + 0.5)
         self.widget.cake_widget.activate_vertical_line()
-        self.plot_cake_integral(x)
+        self.plot_cake_integral(tth)
 
-    def set_image_line_position(self, x):
-        x = self._convert_to_tth(x)
-
-        self.widget.img_widget.set_circle_line(
-            self.model.calibration_model.get_two_theta_array(), np.deg2rad(x))
-
-    def _convert_to_tth(self, x):
-        if self.model.integration_unit == 'q_A^-1':
-            return self.convert_x_value(x, 'q_A^-1', '2th_deg')
-        elif self.model.integration_unit == 'd_A':
-            return self.convert_x_value(x, 'd_A', '2th_deg')
-        return x
+    def set_image_line_position(self, tth):
+        if not self.model.calibration_model.is_calibrated:
+            self.widget.img_widget.deactivate_circle_scatter()
+            return
+        self.widget.img_widget.activate_circle_scatter()
+        self.widget.img_widget.set_circle_line(self.model.calibration_model.get_two_theta_array(), np.deg2rad(tth))
 
     def set_iteration_mode_number(self):
         self.model.img_model.set_file_iteration_mode('number')
@@ -913,6 +930,9 @@ class ImageController(object):
     def set_iteration_mode_time(self):
         self.model.img_model.set_file_iteration_mode('time')
 
+    def select_source(self, source):
+        self.model.img_model.select_source(source)
+
     def convert_x_value(self, value, previous_unit, new_unit):
         wavelength = self.model.calibration_model.wavelength
         if previous_unit == '2th_deg':
@@ -947,6 +967,14 @@ class ImageController(object):
             self.model.calibration_model.load(filename)
             self.widget.calibration_lbl.setText(
                 self.model.calibration_model.calibration_name)
+            self.widget.wavelength_lbl.setText('{:.4f}'.format(self.model.calibration_model.wavelength * 1e10) + ' A')
+            self.model.img_model.img_changed.emit()
+
+    def set_wavelength(self):
+        wavelength, ok = QtWidgets.QInputDialog.getText(self.widget, 'Set Wavelength', 'Wavelength in Angstroms:')
+        if ok:
+            self.model.calibration_model.pattern_geometry.wavelength = float(wavelength) * 1e-10
+            self.widget.wavelength_lbl.setText('{:.4f}'.format(self.model.calibration_model.wavelength * 1e10) + ' A')
             self.model.img_model.img_changed.emit()
 
     def auto_process_cb_click(self):
@@ -999,12 +1027,12 @@ class ImageController(object):
                             out_file.write("{:6.2f}".format(azi) + row_str + '\n')
 
     def update_gui_from_configuration(self):
-        self.widget.img_mask_btn.setChecked(self.model.use_mask)
-        self.widget.mask_transparent_cb.setChecked(self.model.transparent_mask)
-        self.widget.autoprocess_cb.setChecked(self.model.img_model.autoprocess)
+        self.widget.img_mask_btn.setChecked(int(self.model.use_mask))
+        self.widget.mask_transparent_cb.setChecked(bool(self.model.transparent_mask))
+        self.widget.autoprocess_cb.setChecked(bool(self.model.img_model.autoprocess))
         self.widget.calibration_lbl.setText(self.model.calibration_model.calibration_name)
 
-        self.update_img()
+        self.update_img_control_widget()
         self.update_mask_mode()
         self.update_roi_in_gui()
 
@@ -1013,7 +1041,7 @@ class ImageController(object):
         elif not self.model.current_configuration.auto_integrate_cake and self.widget.img_mode == 'Cake':
             self.activate_image_mode()
         elif self.model.current_configuration.auto_integrate_cake and self.widget.img_mode == 'Cake':
-            self._update_cake_line_pos()
+            self.set_cake_line_position(self.model.clicked_tth)
             self._update_cake_mouse_click_pos()
         elif not self.model.current_configuration.auto_integrate_cake and self.widget.img_mode == 'Image':
             self._update_image_line_pos()
