@@ -32,20 +32,24 @@ from pyFAI.calibrant import Calibrant
 from pyFAI.detectors import Detector, ALL_DETECTORS, NexusDetector
 from pyFAI.geometryRefinement import GeometryRefinement
 from pyFAI.massif import Massif
-from qtpy import QtCore
 from skimage.measure import find_contours
 
 from .. import calibrants_path
 from .ImgModel import ImgModel
-from .util.HelperModule import get_base_name, rotate_matrix_p90, rotate_matrix_m90, get_partial_index
-from .util.calc import supersample_image
+from .util import Signal
+from .util.HelperModule import (
+    get_base_name,
+    rotate_matrix_p90,
+    rotate_matrix_m90,
+    get_partial_index,
+)
+from .util.calc import supersample_image, trim_trailing_zeros
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
 
 
-class CalibrationModel(QtCore.QObject):
-    detector_reset = QtCore.Signal()
+class CalibrationModel(object):
 
     def __init__(self, img_model=None):
         """
@@ -58,28 +62,38 @@ class CalibrationModel(QtCore.QObject):
         self.points_index = []
 
         self.detector = Detector(pixel1=79e-6, pixel2=79e-6)
+        # self.detector.shape = (2048, 2048)
         self.detector_mode = DetectorModes.CUSTOM
-        self._original_detector = None  # used for saving original state before rotating or flipping
-        self.pattern_geometry = GeometryRefinement(detector=self.detector, wavelength=0.3344e-10,
-                                                   poni1=0, poni2=0)  # default params are necessary, otherwise fails...
+        self._original_detector = (
+            None  # used for saving original state before rotating or flipping
+        )
+        self.pattern_geometry = GeometryRefinement(
+            detector=self.detector, wavelength=0.3344e-10, poni1=0, poni2=0
+        )  # default params are necessary, otherwise fails...
         self.pattern_geometry_img_shape = None
         self.cake_geometry = None
         self.cake_geometry_img_shape = None
         self.calibrant = Calibrant()
 
-        self.orig_pixel1 = self.detector.pixel1  # needs to be extra stored for applying supersampling
+        self.orig_pixel1 = (
+            self.detector.pixel1
+        )  # needs to be extra stored for applying supersampling
         self.orig_pixel2 = self.detector.pixel2
 
-        self.start_values = {'dist': 200e-3,
-                             'wavelength': 0.3344e-10,
-                             'polarization_factor': 0.99}
+        self.start_values = {
+            "dist": 200e-3,
+            "wavelength": 0.3344e-10,
+            "polarization_factor": 0.99,
+        }
         self.fit_wavelength = False
-        self.fixed_values = {}  # dictionary for fixed parameters during calibration (keys can be e.g. rot1, poni1 etc.
+        self.fixed_values = (
+            {}
+        )  # dictionary for fixed parameters during calibration (keys can be e.g. rot1, poni1 etc.
         # and values are the values to what the respective parameter will be set
         self.is_calibrated = False
         self.use_mask = False
-        self.filename = ''
-        self.calibration_name = 'None'
+        self.filename = ""
+        self.calibration_name = ""
         self.polarization_factor = 0.99
         self.supersampling_factor = 1
         self.correct_solid_angle = True
@@ -97,6 +111,10 @@ class CalibrationModel(QtCore.QObject):
 
         self.peak_search_algorithm = None
 
+        self.img_model.img_changed.connect(self._check_detector_and_image_shape)
+
+        self.detector_reset = Signal()
+
     def find_peaks_automatic(self, x, y, peak_ind):
         """
         Searches peaks by using the Massif algorithm
@@ -109,8 +127,10 @@ class CalibrationModel(QtCore.QObject):
         :return:
             array of points found
         """
-        massif = Massif(self.img_model.img_data)
-        cur_peak_points = massif.find_peaks((int(np.round(x)), int(np.round(y))), stdout=DummyStdOut())
+        massif = Massif(self.img_model.img_data, median_prefilter=False)
+        cur_peak_points = massif.find_peaks(
+            (int(np.round(x)), int(np.round(y))), stdout=DummyStdOut()
+        )
         if len(cur_peak_points):
             self.points.append(np.array(cur_peak_points))
             self.points_index.append(peak_ind)
@@ -137,7 +157,9 @@ class CalibrationModel(QtCore.QObject):
         top_ind = int(np.round(y - search_size * 0.5))
         if top_ind < 0:
             top_ind = 0
-        search_array = self.img_model.img_data[left_ind:(left_ind + search_size), top_ind:(top_ind + search_size)]
+        search_array = self.img_model.img_data[
+            left_ind : (left_ind + search_size), top_ind : (top_ind + search_size)
+        ]
         x_ind, y_ind = np.where(search_array == search_array.max())
         x_ind = x_ind[0] + left_ind
         y_ind = y_ind[0] + top_ind
@@ -151,26 +173,32 @@ class CalibrationModel(QtCore.QObject):
 
     def remove_last_peak(self):
         if self.points:
-            num_points = int(self.points[-1].size / 2)  # each peak is x, y so length is twice as number of peaks
+            num_points = int(
+                self.points[-1].size / 2
+            )  # each peak is x, y so length is twice as number of peaks
             self.points.pop(-1)
             self.points_index.pop(-1)
             return num_points
 
     def create_cake_geometry(self):
-        self.cake_geometry = AzimuthalIntegrator(splineFile=self.distortion_spline_filename)
+        self.cake_geometry = AzimuthalIntegrator(
+            splineFile=self.distortion_spline_filename
+        )
 
         pyFAI_parameter = self.pattern_geometry.getPyFAI()
-        pyFAI_parameter['wavelength'] = self.pattern_geometry.wavelength
+        pyFAI_parameter["wavelength"] = self.pattern_geometry.wavelength
 
-        self.cake_geometry.setPyFAI(dist=pyFAI_parameter['dist'],
-                                    poni1=pyFAI_parameter['poni1'],
-                                    poni2=pyFAI_parameter['poni2'],
-                                    rot1=pyFAI_parameter['rot1'],
-                                    rot2=pyFAI_parameter['rot2'],
-                                    rot3=pyFAI_parameter['rot3'])
+        self.cake_geometry.setPyFAI(
+            dist=pyFAI_parameter["dist"],
+            poni1=pyFAI_parameter["poni1"],
+            poni2=pyFAI_parameter["poni2"],
+            rot1=pyFAI_parameter["rot1"],
+            rot2=pyFAI_parameter["rot2"],
+            rot3=pyFAI_parameter["rot3"],
+        )
 
         self.cake_geometry.detector = self.detector
-        self.cake_geometry.wavelength = pyFAI_parameter['wavelength']
+        self.cake_geometry.wavelength = pyFAI_parameter["wavelength"]
 
     def setup_peak_search_algorithm(self, algorithm, mask=None):
         """
@@ -181,19 +209,24 @@ class CalibrationModel(QtCore.QObject):
             if a mask is used during the process this is provided here as a 2d array for the image.
         """
 
-        if algorithm == 'Massif':
-            self.peak_search_algorithm = Massif(self.img_model.img_data)
-        elif algorithm == 'Blob':
+        if algorithm == "Massif":
+            self.peak_search_algorithm = Massif(
+                self.img_model.img_data, median_prefilter=False
+            )
+        elif algorithm == "Blob":
             if mask is not None:
-                self.peak_search_algorithm = BlobDetection(self.img_model.img_data * mask)
+                self.peak_search_algorithm = BlobDetection(
+                    self.img_model.img_data * mask
+                )
             else:
                 self.peak_search_algorithm = BlobDetection(self.img_model.img_data)
             self.peak_search_algorithm.process()
         else:
             return
 
-    def search_peaks_on_ring(self, ring_index, delta_tth=0.1, min_mean_factor=1,
-                             upper_limit=55000, mask=None):
+    def search_peaks_on_ring(
+        self, ring_index, delta_tth=0.1, min_mean_factor=1, upper_limit=55000, mask=None
+    ):
         """
         This function is searching for peaks on an expected ring. It needs an initial calibration
         before. Then it will search for the ring within some delta_tth and other parameters to get
@@ -221,7 +254,7 @@ class CalibrationModel(QtCore.QObject):
         tth_calibrant_list = self.calibrant.get_2th()
         if ring_index >= len(tth_calibrant_list):
             raise NotEnoughSpacingsInCalibrant()
-        tth_calibrant = np.float(tth_calibrant_list[ring_index])
+        tth_calibrant = float(tth_calibrant_list[ring_index])
 
         # get the calculated two theta values for the whole image
         tth_array = self.pattern_geometry.twoThetaArray(self.img_model.img_data.shape)
@@ -235,7 +268,9 @@ class CalibrationModel(QtCore.QObject):
             mask = ring_mask
 
         # calculate the mean and standard deviation of this area
-        sub_data = np.array(self.img_model.img_data.ravel()[np.where(mask.ravel())], dtype=np.float64)
+        sub_data = np.array(
+            self.img_model.img_data.ravel()[np.where(mask.ravel())], dtype=np.float64
+        )
         sub_data[np.where(sub_data > upper_limit)] = np.NaN
         mean = np.nanmean(sub_data)
         std = np.nanstd(sub_data)
@@ -248,9 +283,12 @@ class CalibrationModel(QtCore.QObject):
 
         keep = int(np.ceil(np.sqrt(size2)))
         try:
+            old_stdout = sys.stdout
             sys.stdout = DummyStdOut
-            res = self.peak_search_algorithm.peaks_from_area(mask2, Imin=mean - std, keep=keep)
-            sys.stdout = sys.__stdout__
+            res = self.peak_search_algorithm.peaks_from_area(
+                mask2, Imin=mean - std, keep=keep
+            )
+            sys.stdout = old_stdout
         except IndexError:
             res = []
 
@@ -269,7 +307,7 @@ class CalibrationModel(QtCore.QObject):
 
     def set_start_values(self, start_values):
         self.start_values = start_values
-        self.polarization_factor = start_values['polarization_factor']
+        self.polarization_factor = start_values["polarization_factor"]
 
     def set_pixel_size(self, pixel_size):
         """
@@ -282,6 +320,10 @@ class CalibrationModel(QtCore.QObject):
         self.detector.pixel2 = self.orig_pixel2
         self.set_supersampling()
 
+    def update_detector_shape(self):
+        self.detector.shape = self.img_model.img_data.shape
+        self.detector.max_shape = self.img_model.img_data.shape
+
     def set_fixed_values(self, fixed_values):
         """
         Sets the fixed and not fitted values for the geometry refinement
@@ -292,28 +334,38 @@ class CalibrationModel(QtCore.QObject):
         self.fixed_values = fixed_values
 
     def calibrate(self):
+        if len(self.points) == 0:
+            raise NoPointsError("No starting points for calibration found.")
+
         self.reset_supersampling()
-        self.pattern_geometry = GeometryRefinement(self.create_point_array(self.points, self.points_index),
-                                                   dist=self.start_values['dist'],
-                                                   wavelength=self.start_values['wavelength'],
-                                                   detector=self.detector,
-                                                   calibrant=self.calibrant,
-                                                   splineFile=self.distortion_spline_filename)
+        self.pattern_geometry = GeometryRefinement(
+            self.create_point_array(self.points, self.points_index),
+            dist=self.start_values["dist"],
+            wavelength=self.start_values["wavelength"],
+            detector=self.detector,
+            calibrant=self.calibrant,
+            splineFile=self.distortion_spline_filename,
+        )
 
         self.refine()
         self.create_cake_geometry()
         self.is_calibrated = True
 
-        self.calibration_name = 'current'
+        self.calibration_name = "current"
         self.set_supersampling()
         # reset the integrator (not the geometric parameters)
         self.pattern_geometry.reset()
 
     def refine(self):
-        self.reset_supersampling()
-        self.pattern_geometry.data = self.create_point_array(self.points, self.points_index)
+        if len(self.points) == 0:
+            raise NoPointsError("No points for refinement found.")
 
-        fix = ['wavelength']
+        self.reset_supersampling()
+        self.pattern_geometry.data = self.create_point_array(
+            self.points, self.points_index
+        )
+
+        fix = ["wavelength"]
         if self.fit_wavelength:
             fix = []
 
@@ -335,6 +387,8 @@ class CalibrationModel(QtCore.QObject):
             if self.detector.shape != self.img_model.img_data.shape:
                 self.reset_detector()
                 self.detector_reset.emit()
+        else:
+            self.reset_detector()
 
     def _prepare_integration_mask(self, mask):
         if mask is None:
@@ -348,16 +402,43 @@ class CalibrationModel(QtCore.QObject):
 
     def _prepare_integration_super_sampling(self, mask):
         if self.supersampling_factor > 1:
-            img_data = supersample_image(self.img_model.img_data, self.supersampling_factor)
+            img_data = supersample_image(
+                self.img_model.img_data, self.supersampling_factor
+            )
             if mask is not None:
                 mask = supersample_image(mask, self.supersampling_factor)
         else:
             img_data = self.img_model.img_data
         return img_data, mask
 
-    def integrate_1d(self, num_points=None, mask=None, polarization_factor=None, filename=None,
-                     unit='2th_deg', method='csr', azi_range=None):
-        if np.sum(mask) == self.img_model.img_data.shape[0] * self.img_model.img_data.shape[1]:
+    def integrate_1d(
+        self,
+        num_points=None,
+        mask=None,
+        polarization_factor=None,
+        filename=None,
+        unit="2th_deg",
+        method="csr",
+        azi_range=None,
+        trim_zeros=True,
+    ):
+        """
+        :param num_points: number of points for the integration
+        :param mask: mask for the integration
+        :param polarization_factor: polarization factor for the integration
+        :param filename: filename for saving the integration
+        :param unit: unit for the integration, possible values are '2th_deg', 'q_A^-1', 'r_mm', 'r_m', 'd_A'
+        :param method: method for the integration, possible values are 'csr', 'splitbbox', 'lut', 'nosplit_csr',
+                          'full_csr', 'numpy', 'cython', 'BBox', 'splitPixel', 'lut_ocl', 'csr_ocl', 'csr_ocl_memsave',
+                            'csr_ocl_lut', 'csr_ocl_lut_memsave', 'csr_numpy', 'csr_numpy_memsave'
+        :param azi_range: azimuthal range for the integration
+        :param trim_zeros: if True, the trailing zeros in the integration will be trimmed
+        :return: tth, intensity
+        """
+        if (
+            np.sum(mask)
+            == self.img_model.img_data.shape[0] * self.img_model.img_data.shape[1]
+        ):
             # do not perform integration if the image is completely masked...
             return self.tth, self.int
 
@@ -380,56 +461,85 @@ class CalibrationModel(QtCore.QObject):
 
         t1 = time.time()
 
-        if unit == 'd_A':
+        if unit == "d_A":
             try:
-                self.tth, self.int = self.pattern_geometry.integrate1d(img_data, num_points,
-                                                                       method=method,
-                                                                       unit='2th_deg',
-                                                                       azimuth_range=azi_range,
-                                                                       mask=mask,
-                                                                       polarization_factor=polarization_factor,
-                                                                       correctSolidAngle=self.correct_solid_angle,
-                                                                       filename=filename)
+                self.tth, self.int = self.pattern_geometry.integrate1d(
+                    img_data,
+                    num_points,
+                    method=method,
+                    unit="2th_deg",
+                    azimuth_range=azi_range,
+                    mask=mask,
+                    polarization_factor=polarization_factor,
+                    correctSolidAngle=self.correct_solid_angle,
+                    filename=filename,
+                )
             except NameError:
-                self.tth, self.int = self.pattern_geometry.integrate1d(img_data, num_points,
-                                                                       method='csr',
-                                                                       unit='2th_deg',
-                                                                       azimuth_range=azi_range,
-                                                                       mask=mask,
-                                                                       polarization_factor=polarization_factor,
-                                                                       correctSolidAngle=self.correct_solid_angle,
-                                                                       filename=filename)
-            self.tth = self.pattern_geometry.wavelength / (2 * np.sin(self.tth / 360 * np.pi)) * 1e10
+                self.tth, self.int = self.pattern_geometry.integrate1d(
+                    img_data,
+                    num_points,
+                    method="csr",
+                    unit="2th_deg",
+                    azimuth_range=azi_range,
+                    mask=mask,
+                    polarization_factor=polarization_factor,
+                    correctSolidAngle=self.correct_solid_angle,
+                    filename=filename,
+                )
+            self.tth = (
+                self.pattern_geometry.wavelength
+                / (2 * np.sin(self.tth / 360 * np.pi))
+                * 1e10
+            )
             self.int = self.int
         else:
             try:
-                self.tth, self.int = self.pattern_geometry.integrate1d(img_data, num_points,
-                                                                       method=method,
-                                                                       unit=unit,
-                                                                       azimuth_range=azi_range,
-                                                                       mask=mask,
-                                                                       polarization_factor=polarization_factor,
-                                                                       correctSolidAngle=self.correct_solid_angle,
-                                                                       filename=filename)
+                self.tth, self.int = self.pattern_geometry.integrate1d(
+                    img_data,
+                    num_points,
+                    method=method,
+                    unit=unit,
+                    azimuth_range=azi_range,
+                    mask=mask,
+                    polarization_factor=polarization_factor,
+                    correctSolidAngle=self.correct_solid_angle,
+                    filename=filename,
+                )
             except NameError:
-                self.tth, self.int = self.pattern_geometry.integrate1d(img_data, num_points,
-                                                                       method='csr',
-                                                                       unit=unit,
-                                                                       azimuth_range=azi_range,
-                                                                       mask=mask,
-                                                                       polarization_factor=polarization_factor,
-                                                                       correctSolidAngle=self.correct_solid_angle,
-                                                                       filename=filename)
-        logger.info('1d integration of {0}: {1}s.'.format(os.path.basename(self.img_model.filename), time.time() - t1))
+                self.tth, self.int = self.pattern_geometry.integrate1d(
+                    img_data,
+                    num_points,
+                    method="csr",
+                    unit=unit,
+                    azimuth_range=azi_range,
+                    mask=mask,
+                    polarization_factor=polarization_factor,
+                    correctSolidAngle=self.correct_solid_angle,
+                    filename=filename,
+                )
+        logger.info(
+            "1d integration of {0}: {1}s.".format(
+                os.path.basename(self.img_model.filename), time.time() - t1
+            )
+        )
+
+        if (
+            np.sum(self.int) != 0 and trim_zeros
+        ):  # only trim zeros if not everything is 0 (e.g. bkg-subtraction of the same image)
+            self.tth, self.int = trim_trailing_zeros(self.tth, self.int)
 
-        ind = np.where((self.int > 0) & (~np.isnan(self.int)))
-        self.tth = self.tth[ind]
-        self.int = self.int[ind]
         return self.tth, self.int
 
-    def integrate_2d(self, mask=None, polarization_factor=None, unit='2th_deg', method='csr',
-                     rad_points=None, azimuth_points=360,
-                     azimuth_range=None):
+    def integrate_2d(
+        self,
+        mask=None,
+        polarization_factor=None,
+        unit="2th_deg",
+        method="csr",
+        rad_points=None,
+        azimuth_points=360,
+        azimuth_range=None,
+    ):
         if polarization_factor is None:
             polarization_factor = self.polarization_factor
 
@@ -448,14 +558,22 @@ class CalibrationModel(QtCore.QObject):
 
         t1 = time.time()
 
-        res = self.cake_geometry.integrate2d(img_data, rad_points, azimuth_points,
-                                             azimuth_range=azimuth_range,
-                                             method=method,
-                                             mask=mask,
-                                             unit=unit,
-                                             polarization_factor=polarization_factor,
-                                             correctSolidAngle=self.correct_solid_angle)
-        logger.info('2d integration of {0}: {1}s.'.format(os.path.basename(self.img_model.filename), time.time() - t1))
+        res = self.cake_geometry.integrate2d(
+            img_data,
+            rad_points,
+            azimuth_points,
+            azimuth_range=azimuth_range,
+            method=method,
+            mask=mask,
+            unit=unit,
+            polarization_factor=polarization_factor,
+            correctSolidAngle=self.correct_solid_angle,
+        )
+        logger.info(
+            "2d integration of {0}: {1}s.".format(
+                os.path.basename(self.img_model.filename), time.time() - t1
+            )
+        )
         self.cake_img = res[0]
         self.cake_tth = res[1]
         self.cake_azi = res[2]
@@ -469,12 +587,15 @@ class CalibrationModel(QtCore.QObject):
         :return: cake_azimuth_pixel, intensity
         """
         tth_partial_index = get_partial_index(self.cake_tth, tth)
+        if tth_partial_index is None:
+            return [], []
+
         tth_center = tth_partial_index + 0.5
         left = tth_center - 0.5 * bins
         right = tth_center + 0.5 * bins
 
         y1 = abs(np.ceil(left) - left) * self.cake_img[:, int(np.floor(left))]
-        y2 = np.sum(self.cake_img[:, int(np.ceil(left)): int(np.floor(right))], axis=1)
+        y2 = np.sum(self.cake_img[:, int(np.ceil(left)) : int(np.floor(right))], axis=1)
         y3 = (right - np.floor(right)) * self.cake_img[:, int(np.floor(right))]
 
         x = np.array(range(len(self.cake_azi))) + 0.5
@@ -496,16 +617,16 @@ class CalibrationModel(QtCore.QObject):
 
     def get_calibration_parameter(self):
         pyFAI_parameter = self.pattern_geometry.getPyFAI()
-        pyFAI_parameter['polarization_factor'] = self.polarization_factor
+        pyFAI_parameter["polarization_factor"] = self.polarization_factor
         try:
             fit2d_parameter = self.pattern_geometry.getFit2D()
-            fit2d_parameter['polarization_factor'] = self.polarization_factor
+            fit2d_parameter["polarization_factor"] = self.polarization_factor
         except TypeError:
             fit2d_parameter = None
 
-        pyFAI_parameter['wavelength'] = self.pattern_geometry.wavelength
+        pyFAI_parameter["wavelength"] = self.pattern_geometry.wavelength
         if fit2d_parameter:
-            fit2d_parameter['wavelength'] = self.pattern_geometry.wavelength
+            fit2d_parameter["wavelength"] = self.pattern_geometry.wavelength
 
         return pyFAI_parameter, fit2d_parameter
 
@@ -513,8 +634,8 @@ class CalibrationModel(QtCore.QObject):
         # calculates the number of points for an integrated pattern, based on the distance of the beam center to the the
         # image corners. Maximum value is determined by the shape of the image.
         fit2d_parameter = self.pattern_geometry.getFit2D()
-        center_x = fit2d_parameter['centerX']
-        center_y = fit2d_parameter['centerY']
+        center_x = fit2d_parameter["centerX"]
+        center_y = fit2d_parameter["centerY"]
         width, height = img_shape
 
         if width > center_x > 0:
@@ -526,23 +647,28 @@ class CalibrationModel(QtCore.QObject):
             side2 = np.max([abs(height - center_y), center_y])
         else:
             side2 = height
-        max_dist = np.sqrt(side1 ** 2 + side2 ** 2)
+        max_dist = np.sqrt(side1**2 + side2**2)
         return int(max_dist * max_dist_factor)
 
     def load(self, filename):
         """
-        Loads a calibration file and and sets all the calibration parameter.
+        Loads a calibration file andsets all the calibration parameter.
         :param filename: filename for a *.poni calibration file
         """
-        self.pattern_geometry = GeometryRefinement(wavelength=0.3344e-10, detector=self.detector,
-                                                   poni1=0, poni2=0)  # default params are necessary, otherwise fails...
+        self.pattern_geometry = GeometryRefinement(
+            wavelength=0.3344e-10, detector=self.detector, poni1=0, poni2=0
+        )  # default params are necessary, otherwise fails...
         self.pattern_geometry.load(filename)
         self.orig_pixel1 = self.pattern_geometry.pixel1
         self.orig_pixel2 = self.pattern_geometry.pixel2
 
-        if self.pattern_geometry.pixel1 == self.detector.pixel1 and \
-                self.pattern_geometry.pixel2 == self.detector.pixel2:
-            self.pattern_geometry.detector = self.detector  # necessary since loading a poni file will reset the detector
+        if (
+            self.pattern_geometry.pixel1 == self.detector.pixel1
+            and self.pattern_geometry.pixel2 == self.detector.pixel2
+        ):
+            self.pattern_geometry.detector = (
+                self.detector
+            )  # necessary since loading a poni file will reset the detector
         else:
             self.reset_detector()
 
@@ -573,7 +699,7 @@ class CalibrationModel(QtCore.QObject):
         self._load_detector(NexusDetector(filename))
 
     def _load_detector(self, detector):
-        """ Loads a pyFAI detector
+        """Loads a pyFAI detector
         :param detector: an instance of pyFAI Detector
         """
         self.detector = detector
@@ -591,7 +717,10 @@ class CalibrationModel(QtCore.QObject):
 
     def reset_detector(self):
         self.detector_mode = DetectorModes.CUSTOM
-        self.detector = Detector(pixel1=self.orig_pixel1, pixel2=self.orig_pixel2)
+        self.detector = Detector(
+            pixel1=self.detector.pixel1, pixel2=self.detector.pixel2
+        )
+        self.update_detector_shape()
         self.pattern_geometry.detector = self.detector
         if self.cake_geometry:
             self.cake_geometry.detector = self.detector
@@ -600,10 +729,13 @@ class CalibrationModel(QtCore.QObject):
     def create_file_header(self):
         try:
             # pyFAI version 0.12.0
-            return self.pattern_geometry.makeHeaders(polarization_factor=self.polarization_factor)
+            return self.pattern_geometry.makeHeaders(
+                polarization_factor=self.polarization_factor
+            )
         except AttributeError:
             # pyFAI after version 0.12.0
             from pyFAI.io import DefaultAiWriter
+
             return DefaultAiWriter(None, self.pattern_geometry).make_headers()
 
     def set_fit2d(self, fit2d_parameter):
@@ -612,22 +744,24 @@ class CalibrationModel(QtCore.QObject):
         'directDist', 'centerX', 'centerY', 'tilt', 'tiltPlanRotation', 'pixelX', pixelY',
         'polarization_factor', 'wavelength'
         """
-        self.pattern_geometry.setFit2D(directDist=fit2d_parameter['directDist'],
-                                       centerX=fit2d_parameter['centerX'],
-                                       centerY=fit2d_parameter['centerY'],
-                                       tilt=fit2d_parameter['tilt'],
-                                       tiltPlanRotation=fit2d_parameter['tiltPlanRotation'],
-                                       pixelX=fit2d_parameter['pixelX'],
-                                       pixelY=fit2d_parameter['pixelY'])
+        self.pattern_geometry.setFit2D(
+            directDist=fit2d_parameter["directDist"],
+            centerX=fit2d_parameter["centerX"],
+            centerY=fit2d_parameter["centerY"],
+            tilt=fit2d_parameter["tilt"],
+            tiltPlanRotation=fit2d_parameter["tiltPlanRotation"],
+            pixelX=fit2d_parameter["pixelX"],
+            pixelY=fit2d_parameter["pixelY"],
+        )
         # the detector pixel1 and pixel2 values are updated by setPyFAI
         self.orig_pixel1 = self.detector.pixel1
         self.orig_pixel2 = self.detector.pixel2
 
-        self.pattern_geometry.wavelength = fit2d_parameter['wavelength']
+        self.pattern_geometry.wavelength = fit2d_parameter["wavelength"]
         self.create_cake_geometry()
-        self.polarization_factor = fit2d_parameter['polarization_factor']
-        self.orig_pixel1 = fit2d_parameter['pixelX'] * 1e-6
-        self.orig_pixel2 = fit2d_parameter['pixelY'] * 1e-6
+        self.polarization_factor = fit2d_parameter["polarization_factor"]
+        self.orig_pixel1 = fit2d_parameter["pixelX"] * 1e-6
+        self.orig_pixel2 = fit2d_parameter["pixelY"] * 1e-6
         self.is_calibrated = True
         self.set_supersampling()
 
@@ -637,21 +771,23 @@ class CalibrationModel(QtCore.QObject):
         'dist', 'poni1', 'poni2', 'rot1', 'rot2', 'rot3', 'pixel1', 'pixel2', 'wavelength',
         'polarization_factor'
         """
-        self.pattern_geometry.setPyFAI(dist=pyFAI_parameter['dist'],
-                                       poni1=pyFAI_parameter['poni1'],
-                                       poni2=pyFAI_parameter['poni2'],
-                                       rot1=pyFAI_parameter['rot1'],
-                                       rot2=pyFAI_parameter['rot2'],
-                                       rot3=pyFAI_parameter['rot3'],
-                                       pixel1=pyFAI_parameter['pixel1'],
-                                       pixel2=pyFAI_parameter['pixel2'])
-        self.detector.pixel1 = pyFAI_parameter['pixel1']
-        self.detector.pixel2 = pyFAI_parameter['pixel2']
+        self.pattern_geometry.setPyFAI(
+            dist=pyFAI_parameter["dist"],
+            poni1=pyFAI_parameter["poni1"],
+            poni2=pyFAI_parameter["poni2"],
+            rot1=pyFAI_parameter["rot1"],
+            rot2=pyFAI_parameter["rot2"],
+            rot3=pyFAI_parameter["rot3"],
+            pixel1=pyFAI_parameter["pixel1"],
+            pixel2=pyFAI_parameter["pixel2"],
+        )
+        self.detector.pixel1 = pyFAI_parameter["pixel1"]
+        self.detector.pixel2 = pyFAI_parameter["pixel2"]
         self.orig_pixel1 = self.detector.pixel1
         self.orig_pixel2 = self.detector.pixel2
-        self.pattern_geometry.wavelength = pyFAI_parameter['wavelength']
+        self.pattern_geometry.wavelength = pyFAI_parameter["wavelength"]
         self.create_cake_geometry()
-        self.polarization_factor = pyFAI_parameter['polarization_factor']
+        self.polarization_factor = pyFAI_parameter["polarization_factor"]
         self.is_calibrated = True
         self.set_supersampling()
 
@@ -708,10 +844,16 @@ class CalibrationModel(QtCore.QObject):
 
         :return  : two theta in radians
         """
+        if not isinstance(x, np.ndarray):
+            x = np.array([x])
+        if not isinstance(y, np.ndarray):
+            y = np.array([y])
         x *= self.supersampling_factor
         y *= self.supersampling_factor
 
-        return self.pattern_geometry.tth(x - 0.5, y - 0.5)[0]  # deletes 0.5 because tth function uses pixel indices
+        return self.pattern_geometry.tth(x - 0.5, y - 0.5)[
+            0
+        ]  # deletes 0.5 because tth function uses pixel indices
 
     def get_azi_img(self, x, y):
         """
@@ -723,13 +865,19 @@ class CalibrationModel(QtCore.QObject):
 
         :return  : azimuth in radians
         """
+        # if float convert to np.array:
+        if not isinstance(x, np.ndarray):
+            x = np.array([x])
+        if not isinstance(y, np.ndarray):
+            y = np.array([y])
         x *= self.supersampling_factor
         y *= self.supersampling_factor
         return self.pattern_geometry.chi(x - 0.5, y - 0.5)[0]
 
     def get_two_theta_array(self):
-        return self.pattern_geometry.twoThetaArray(self.img_model.img_data.shape)[::self.supersampling_factor,
-               ::self.supersampling_factor]
+        return self.pattern_geometry.twoThetaArray(self.img_model.img_data.shape)[
+            :: self.supersampling_factor, :: self.supersampling_factor
+        ]
 
     def get_pixel_ind(self, tth, azi):
         """
@@ -741,7 +889,6 @@ class CalibrationModel(QtCore.QObject):
         :return:
             tuple of index 1 and 2
         """
-
         tth_ind = find_contours(self.pattern_geometry.ttha, tth)
         if len(tth_ind) == 0:
             return []
@@ -773,8 +920,7 @@ class CalibrationModel(QtCore.QObject):
         self._transform_pixel_corners(rotate_matrix_m90)
 
     def rotate_detector_p90(self):
-        """
-        """
+        """ """
         self._save_original_detector_definition()
 
         self.swap_detector_shape()
@@ -803,7 +949,7 @@ class CalibrationModel(QtCore.QObject):
         self._original_detector = None
 
     def load_transformations_string_list(self, transformations):
-        """ Transforms the detector parameters (shape, pixel size and distortion correction) based on a
+        """Transforms the detector parameters (shape, pixel size and distortion correction) based on a
         list of transformation actions.
         :param transformations: list of transformations specified as strings, values are "flipud", "fliplr",
                                 "rotate_matrix_m90", "rotate_matrix_p90
@@ -821,7 +967,7 @@ class CalibrationModel(QtCore.QObject):
     def _save_original_detector_definition(self):
         """
         Saves the state of the detector to _original_detector if not done yet. Used for restoration upon resetting
-        the transfromations.
+        the transformations.
         """
         if self._original_detector is None:
             self._original_detector = deepcopy(self.detector)
@@ -833,13 +979,10 @@ class CalibrationModel(QtCore.QObject):
         """
         :param transform_function: function pointer which will affect the dx, dy and pixel corners of the detector
         """
-        if self.detector._dx is not None:
-            old_dx, old_dy = self.detector._dx, self.detector._dy
-            self.detector.set_dx(transform_function(old_dx))
-            self.detector.set_dy(transform_function(old_dy))
-
         if self.detector._pixel_corners is not None:
-            self.detector._pixel_corners = transform_function(self.detector._pixel_corners)
+            self.detector._pixel_corners = np.ascontiguousarray(
+                transform_function(self.detector.get_pixel_corners())
+            )
 
     def _swap_pixel_size(self):
         """swaps the pixel sizes"""
@@ -851,17 +994,26 @@ class CalibrationModel(QtCore.QObject):
         if self.detector.shape is not None:
             self.detector.shape = (self.detector.shape[1], self.detector.shape[0])
         if self.detector.max_shape is not None:
-            self.detector.max_shape = (self.detector.max_shape[1], self.detector.max_shape[0])
+            self.detector.max_shape = (
+                self.detector.max_shape[1],
+                self.detector.max_shape[0],
+            )
 
     def _swap_detector_module_size(self):
         """swaps the module size and gap sizes for e.g. Pilatus Detectors"""
-        if hasattr(self.detector, 'module_size'):
-            self.detector.module_size = (self.detector.module_size[1], self.detector.module_size[0])
-        if hasattr(self.detector, 'MODULE_GAP'):
-            self.detector.MODULE_GAP = (self.detector.MODULE_GAP[1], self.detector.MODULE_GAP[0])
+        if hasattr(self.detector, "module_size"):
+            self.detector.module_size = (
+                self.detector.module_size[1],
+                self.detector.module_size[0],
+            )
+        if hasattr(self.detector, "MODULE_GAP"):
+            self.detector.MODULE_GAP = (
+                self.detector.MODULE_GAP[1],
+                self.detector.MODULE_GAP[0],
+            )
 
     def _reset_detector_mask(self):
-        """resets and recalculates the mask. Transforamtions to shape and module size have to be performed before."""
+        """resets and recalculates the mask. Transformations to shape and module size have to be performed before."""
         self.detector._mask = False
 
 
@@ -879,6 +1031,10 @@ class DetectorShapeError(Exception):
     pass
 
 
+class NoPointsError(Exception):
+    pass
+
+
 class DummyStdOut(object):
     @classmethod
     def write(cls, *args, **kwargs):
@@ -903,7 +1059,7 @@ def get_available_detectors():
     detector_names_sorted = [detector_names[i] for i in sorted_indices]
     detector_classes_sorted = [list(detector_classes)[i] for i in sorted_indices]
 
-    base_class_index = detector_names_sorted.index('Detector')
+    base_class_index = detector_names_sorted.index("Detector")
     del detector_names_sorted[base_class_index]
     del detector_classes_sorted[base_class_index]
 
