@@ -19,32 +19,35 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-from scipy.interpolate import interp1d, interp2d
+from scipy.interpolate import interp2d
 import numpy as np
-from qtpy import QtCore
-
 import h5py
 
+from xypattern import Pattern
+from xypattern.combine import stitch_patterns
+
+from .util import Signal
 from .util import jcpds
-from .util import Pattern
 from .Configuration import Configuration
-from . import ImgModel, CalibrationModel, MaskModel, PhaseModel, PatternModel, OverlayModel
+from . import (
+    ImgModel,
+    CalibrationModel,
+    MaskModel,
+    PhaseModel,
+    PatternModel,
+    OverlayModel,
+    MapModel,
+    BatchModel,
+)
+from .MapModel2 import MapModel2
 from .. import __version__
 
 
-class DioptasModel(QtCore.QObject):
+class DioptasModel(object):
     """
     Handles all the data used in Dioptas. Image, Calibration and Mask are handled by so called configurations.
     Patterns and overlays are global and always the same, no matter which configuration is selected.
     """
-    configuration_added = QtCore.Signal()
-    configuration_selected = QtCore.Signal(int)  # new index
-    configuration_removed = QtCore.Signal(int)  # removed index
-
-    img_changed = QtCore.Signal()
-    pattern_changed = QtCore.Signal()
-    cake_changed = QtCore.Signal()
-    enabled_phases_in_cake = QtCore.Signal()
 
     def __init__(self):
         super(DioptasModel, self).__init__()
@@ -59,6 +62,23 @@ class DioptasModel(QtCore.QObject):
         self._combine_cakes = False
         self._cake_data = None
 
+        self.configuration_added = Signal()
+        self.configuration_selected = Signal(int)  # new index
+        self.configuration_removed = Signal(int)  # removed index
+
+        self.img_changed = Signal()
+        self.pattern_changed = Signal()
+        self.cake_changed = Signal()
+        self.enabled_phases_in_cake = Signal()
+
+        self.clicked_tth = 0
+        self.clicked_azi = 0
+
+        self.clicked_tth_changed = Signal()
+        self.clicked_azi_changed = Signal()
+        self.clicked_tth_changed.connect(self.update_clicked_tth)
+        self.clicked_azi_changed.connect(self.update_clicked_azi)
+
         self.connect_models()
 
     def add_configuration(self):
@@ -69,15 +89,19 @@ class DioptasModel(QtCore.QObject):
         self.configurations.append(Configuration(self.working_directories))
 
         if self.current_configuration.calibration_model.is_calibrated:
-            dioptas_config_folder = os.path.join(os.path.expanduser('~'), '.Dioptas')
+            dioptas_config_folder = os.path.join(os.path.expanduser("~"), ".Dioptas")
             if not os.path.isdir(dioptas_config_folder):
                 os.mkdir(dioptas_config_folder)
             self.current_configuration.calibration_model.save(
-                os.path.join(dioptas_config_folder, 'transfer.poni'))
+                os.path.join(dioptas_config_folder, "transfer.poni")
+            )
             self.configurations[-1].calibration_model.load(
-                os.path.join(dioptas_config_folder, 'transfer.poni'))
+                os.path.join(dioptas_config_folder, "transfer.poni")
+            )
 
-        self.configurations[-1].img_model._img_data = self.current_configuration.img_model.img_data
+        self.configurations[-1].img_model._img_data = (
+            self.current_configuration.img_model.img_data
+        )
 
         self.select_configuration(len(self.configurations) - 1)
         self.configuration_added.emit()
@@ -101,52 +125,55 @@ class DioptasModel(QtCore.QObject):
         Saves the current state of the model in a h5py file. file-ending can be chosen as wanted. Usually Dioptas
         projects are saved as *.dio files.
         """
-        f = h5py.File(filename, 'w')
+        f = h5py.File(filename, "w")
 
-        f.attrs['__version__'] = __version__
+        f.attrs["__version__"] = __version__
 
         # save configuration
-        configurations_group = f.create_group('configurations')
-        configurations_group.attrs['selected_configuration'] = self.configuration_ind
+        configurations_group = f.create_group("configurations")
+        configurations_group.attrs["selected_configuration"] = self.configuration_ind
         for ind, configuration in enumerate(self.configurations):
             configuration_group = configurations_group.create_group(str(ind))
             configuration.save_in_hdf5(configuration_group)
 
         # save overlays
-        overlay_group = f.create_group('overlays')
+        overlay_group = f.create_group("overlays")
 
         for ind, overlay in enumerate(self.overlay_model.overlays):
-            ov = overlay_group.create_group(str(ind).zfill(5)) # need to fill the ind string, in order to keep it
-                                                               # ordered also for larger numbers of overlays
-            ov.attrs['name'] = overlay.name
-            ov.create_dataset('x', overlay.original_x.shape, 'f', overlay.original_x)
-            ov.create_dataset('y', overlay.original_y.shape, 'f', overlay.original_y)
-            ov.attrs['scaling'] = overlay.scaling
-            ov.attrs['offset'] = overlay.offset
+            ov = overlay_group.create_group(
+                str(ind).zfill(5)
+            )  # need to fill the ind string, in order to keep it
+            # ordered also for larger numbers of overlays
+            ov.attrs["name"] = overlay.name
+            x, y = overlay.original_data
+            ov.create_dataset("x", x.shape, "f", x)
+            ov.create_dataset("y", y.shape, "f", y)
+            ov.attrs["scaling"] = overlay.scaling
+            ov.attrs["offset"] = overlay.offset
 
         # save phases
-        phases_group = f.create_group('phases')
+        phases_group = f.create_group("phases")
         for ind, phase in enumerate(self.phase_model.phases):
             phase_group = phases_group.create_group(str(ind))
-            phase_group.attrs['name'] = phase._name
-            phase_group.attrs['filename'] = phase._filename
-            phase_parameter_group = phase_group.create_group('params')
+            phase_group.attrs["name"] = phase._name
+            phase_group.attrs["filename"] = phase._filename
+            phase_parameter_group = phase_group.create_group("params")
             for key in phase.params:
-                if key == 'comments':
-                    phases_comments_group = phase_group.create_group('comments')
-                    for ind, comment in enumerate(phase.params['comments']):
+                if key == "comments":
+                    phases_comments_group = phase_group.create_group("comments")
+                    for ind, comment in enumerate(phase.params["comments"]):
                         phases_comments_group.attrs[str(ind)] = comment
                 else:
                     phase_parameter_group.attrs[key] = phase.params[key]
-            phase_reflections_group = phase_group.create_group('reflections')
+            phase_reflections_group = phase_group.create_group("reflections")
             for ind, reflection in enumerate(phase.reflections):
                 phase_reflection_group = phase_reflections_group.create_group(str(ind))
-                phase_reflection_group.attrs['d0'] = reflection.d0
-                phase_reflection_group.attrs['d'] = reflection.d
-                phase_reflection_group.attrs['intensity'] = reflection.intensity
-                phase_reflection_group.attrs['h'] = reflection.h
-                phase_reflection_group.attrs['k'] = reflection.k
-                phase_reflection_group.attrs['l'] = reflection.l
+                phase_reflection_group.attrs["d0"] = reflection.d0
+                phase_reflection_group.attrs["d"] = reflection.d
+                phase_reflection_group.attrs["intensity"] = reflection.intensity
+                phase_reflection_group.attrs["h"] = reflection.h
+                phase_reflection_group.attrs["k"] = reflection.k
+                phase_reflection_group.attrs["l"] = reflection.l
         f.flush()
         f.close()
 
@@ -156,44 +183,64 @@ class DioptasModel(QtCore.QObject):
         """
         self.disconnect_models()
 
-        f = h5py.File(filename, 'r')
+        f = h5py.File(filename, "r")
+
+        # delete old configurations
+        for config in self.configurations:
+            del config.img_model
+            del config.calibration_model
+            del config.mask_model
+            import gc
+
+            gc.collect()
 
         # load_configurations
         self.configurations = []
-        for ind, configuration_group in f.get('configurations').items():
+        for ind, configuration_group in f.get("configurations").items():
             configuration = Configuration()
             configuration.load_from_hdf5(configuration_group)
             self.configurations.append(configuration)
-        self.configuration_ind = f.get('configurations').attrs['selected_configuration']
+        self.configuration_ind = f.get("configurations").attrs["selected_configuration"]
 
         self.connect_models()
         self.configuration_added.emit()
         self.select_configuration(self.configuration_ind)
 
         # load phase model
-        for ind, phase_group in f.get('phases').items():
+        for ind, phase_group in f.get("phases").items():
             new_jcpds = jcpds()
-            new_jcpds.name = phase_group.attrs.get('name')
-            new_jcpds.filename = phase_group.attrs.get('filename')
-            for p_key, p_value in phase_group.get('params').attrs.items():
+            new_jcpds.name = phase_group.attrs.get("name")
+            new_jcpds.filename = phase_group.attrs.get("filename")
+            for p_key, p_value in phase_group.get("params").attrs.items():
                 new_jcpds.params[p_key] = p_value
-            for c_key, comment in phase_group.get('comments').attrs.items():
-                new_jcpds.params['comments'].append(comment)
-            for r_key, reflection in phase_group.get('reflections').items():
-                new_jcpds.add_reflection(reflection.attrs['h'], reflection.attrs['k'], reflection.attrs['l'],
-                                         reflection.attrs['intensity'], reflection.attrs['d'])
-            new_jcpds.params['modified'] = bool(phase_group.get('params').attrs['modified'])
+            for c_key, comment in phase_group.get("comments").attrs.items():
+                new_jcpds.params["comments"].append(comment)
+            for r_key, reflection in phase_group.get("reflections").items():
+                new_jcpds.add_reflection(
+                    reflection.attrs["h"],
+                    reflection.attrs["k"],
+                    reflection.attrs["l"],
+                    reflection.attrs["intensity"],
+                    reflection.attrs["d"],
+                )
+            new_jcpds.params["modified"] = bool(
+                phase_group.get("params").attrs["modified"]
+            )
             self.phase_model.phase_files.append(new_jcpds.filename)
             self.phase_model.add_jcpds_object(new_jcpds)
 
         # load overlay model
-        for ind, overlay_group in f.get('overlays').items():
-            self.overlay_model.add_overlay(overlay_group.get('x')[...],
-                                           overlay_group.get('y')[...],
-                                           overlay_group.attrs['name'])
+        for ind, overlay_group in f.get("overlays").items():
+            self.overlay_model.add_overlay(
+                overlay_group.get("x")[...],
+                overlay_group.get("y")[...],
+                overlay_group.attrs["name"],
+            )
             index = len(self.overlay_model.overlays) - 1
-            self.overlay_model.set_overlay_offset(index, overlay_group.attrs['offset'])
-            self.overlay_model.set_overlay_scaling(index, overlay_group.attrs['scaling'])
+            self.overlay_model.set_overlay_offset(index, overlay_group.attrs["offset"])
+            self.overlay_model.set_overlay_scaling(
+                index, overlay_group.attrs["scaling"]
+            )
 
         f.close()
 
@@ -229,7 +276,7 @@ class DioptasModel(QtCore.QObject):
         """
         Connects signals of the currently selected configuration
         """
-        self.img_model.img_changed.connect(self.img_changed)
+        self.img_model.img_changed.connect(self.img_changed, priority=True)
         self.pattern_model.pattern_changed.connect(self.pattern_changed)
         self.current_configuration.cake_changed.connect(self.cake_changed)
 
@@ -242,56 +289,43 @@ class DioptasModel(QtCore.QObject):
         self.current_configuration.working_directories = new
 
     @property
-    def current_configuration(self):
-        """
-        :rtype: Configuration
-        """
+    def current_configuration(self) -> Configuration:
         return self.configurations[self.configuration_ind]
 
     @property
-    def img_model(self):
-        """
-        :rtype: ImgModel
-        """
+    def img_model(self) -> ImgModel:
         return self.configurations[self.configuration_ind].img_model
 
     @property
-    def mask_model(self):
-        """
-        :rtype: MaskModel
-        """
+    def mask_model(self) -> MaskModel:
         return self.configurations[self.configuration_ind].mask_model
 
     @property
-    def calibration_model(self):
-        """
-        :rtype: CalibrationModel
-        """
+    def calibration_model(self) -> CalibrationModel:
         return self.configurations[self.configuration_ind].calibration_model
 
     @property
-    def pattern_model(self):
-        """
-        :rtype: PatternModel
-        """
+    def pattern_model(self) -> PatternModel:
         return self.configurations[self.configuration_ind].pattern_model
 
     @property
-    def overlay_model(self):
-        """
-        :rtype: OverlayModel
-        """
+    def overlay_model(self) -> OverlayModel:
         return self._overlay_model
 
     @property
-    def phase_model(self):
-        """
-        :rtype: PhaseModel
-        """
+    def phase_model(self) -> PhaseModel:
         return self._phase_model
 
     @property
-    def use_mask(self):
+    def batch_model(self) -> BatchModel:
+        return self.configurations[self.configuration_ind].batch_model
+
+    @property
+    def map_model(self) -> MapModel2:
+        return self.configurations[self.configuration_ind].map_model
+
+    @property
+    def use_mask(self) -> bool:
         return self.configurations[self.configuration_ind].use_mask
 
     @use_mask.setter
@@ -307,7 +341,7 @@ class DioptasModel(QtCore.QObject):
         self.configurations[self.configuration_ind].transparent_mask = new_val
 
     @property
-    def integration_unit(self):
+    def integration_unit(self) -> str:
         return self.current_configuration.integration_unit
 
     @integration_unit.setter
@@ -315,11 +349,11 @@ class DioptasModel(QtCore.QObject):
         self.current_configuration.integration_unit = new_val
 
     @property
-    def img_data(self):
+    def img_data(self) -> np.ndarray:
         return self.img_model.img_data
 
     @property
-    def cake_data(self):
+    def cake_data(self) -> np.ndarray:
         if not self.combine_cakes:
             return self.calibration_model.cake_img
         else:
@@ -335,10 +369,12 @@ class DioptasModel(QtCore.QObject):
         combined_tth, combined_azi = np.meshgrid(tth, azi)
         combined_intensity = np.zeros(combined_azi.shape)
         for configuration in self.configurations:
-            cake_interp2d = interp2d(configuration.calibration_model.cake_tth,
-                                     configuration.calibration_model.cake_azi,
-                                     configuration.calibration_model.cake_img,
-                                     fill_value=0)
+            cake_interp2d = interp2d(
+                configuration.calibration_model.cake_tth,
+                configuration.calibration_model.cake_azi,
+                configuration.calibration_model.cake_img,
+                fill_value=0,
+            )
             combined_intensity += cake_interp2d(tth, azi)
         self._cake_data = combined_intensity
 
@@ -408,46 +444,19 @@ class DioptasModel(QtCore.QObject):
             return self._get_combined_cake_azi()
 
     @property
-    def pattern(self):
+    def pattern(self) -> Pattern:
         if not self.combine_patterns:
             return self.pattern_model.pattern
         else:
-            x_min = []
-            for ind in range(0, len(self.configurations)):
-                # determine ranges
-                x = self.configurations[ind].pattern_model.pattern.x
-                x_min.append(np.min(x))
-
-            sorted_pattern_ind = np.argsort(x_min)
-
-            pattern = self.configurations[sorted_pattern_ind[0]].pattern_model.pattern
-            for ind in sorted_pattern_ind[1:]:
-                x1, y1 = pattern.data
-                x2, y2 = self.configurations[ind].pattern_model.pattern.data
-
-                pattern2_interp1d = interp1d(x2, y2, kind='linear')
-
-                overlap_ind_pattern1 = np.where((x1 <= np.max(x2)) & (x1 >= np.min(x2)))[0]
-                left_ind_pattern1 = np.where((x1 <= np.min(x2)))[0]
-                right_ind_pattern2 = np.where((x2 >= np.max(x1)))[0]
-
-                combined_x1 = x1[left_ind_pattern1]
-                combined_y1 = y1[left_ind_pattern1]
-                combined_x2 = x1[overlap_ind_pattern1]
-                combined_y2 = (y1[overlap_ind_pattern1] + pattern2_interp1d(combined_x2)) / 2
-                combined_x3 = x2[right_ind_pattern2]
-                combined_y3 = y2[right_ind_pattern2]
-
-                combined_x = np.hstack((combined_x1, combined_x2, combined_x3))
-                combined_y = np.hstack((combined_y1, combined_y2, combined_y3))
-
-                pattern = Pattern(combined_x, combined_y)
-
-            pattern.name = "Combined Pattern"
-            return pattern
+            return stitch_patterns(
+                [
+                    configuration.pattern_model.pattern
+                    for configuration in self.configurations
+                ]
+            )
 
     @property
-    def combine_patterns(self):
+    def combine_patterns(self) -> bool:
         return self._combine_patterns
 
     @combine_patterns.setter
@@ -455,8 +464,18 @@ class DioptasModel(QtCore.QObject):
         self._combine_patterns = new_val
         self.pattern_changed.emit()
 
+    def save_combined_pattern(self, filename):
+        """
+        Saves the current integrated pattern
+        :param filename: where to save the file
+        """
+        self.pattern.save(filename, unit=self.integration_unit)
+
     @property
     def combine_cakes(self):
+        """
+        :rtype: bool
+        """
         return self._combine_cakes
 
     @combine_cakes.setter
@@ -568,3 +587,15 @@ class DioptasModel(QtCore.QObject):
         for configuration in self.configurations:
             configuration.img_model.load_previous_folder(mec_mode=mec_mode)
         self._teardown_multiple_file_loading()
+
+    def blockSignals(self, block=True):
+        for member in vars(self):
+            attr = getattr(self, member)
+            if isinstance(attr, Signal):
+                attr.blocked = block
+
+    def update_clicked_tth(self, tth):
+        self.clicked_tth = tth
+
+    def update_clicked_azi(self, azi):
+        self.clicked_azi = azi
