@@ -18,7 +18,5 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from .BackgroundExtraction import extract_background
-
-from .Pattern import Pattern
 from .jcpds import jcpds
+from .signal import Signal
