@@ -18,16 +18,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import absolute_import, print_function
-
 import pyqtgraph as pg
 import numpy as np
 from qtpy import QtCore
 from pyqtgraph.exporters.ImageExporter import ImageExporter
 from pyqtgraph.exporters.SVGExporter import SVGExporter
 
+from xypattern import Pattern
+
+from dioptas.model.OverlayModel import Overlay
+
 from .ExLegendItem import LegendItem
-from ...model.util.HelperModule import calculate_color
 
 
 class PatternWidget(QtCore.QObject):
@@ -50,35 +51,57 @@ class PatternWidget(QtCore.QObject):
         self.phases_vlines = []
 
         self.overlays = []
-        self.overlay_names = []
-        self.overlay_show = []
 
-        self.plot_name = ''
+        self.plot_name = ""
 
     def create_graphics(self):
-        self.pattern_plot = self.pg_layout.addPlot(labels={'left': 'Intensity', 'bottom': '2 Theta'})
-        self.pattern_plot.setLabel('bottom', u'2θ', u'°')
+        self.pattern_plot = self.pg_layout.addPlot(
+            labels={"left": "Intensity", "bottom": "2 Theta"}
+        )
+        self.pattern_plot.setLabel("bottom", "2θ", "°")
         self.pattern_plot.enableAutoRange(False)
         self.pattern_plot.buttonsHidden = True
         self.view_box = self.pattern_plot.vb
-        self.legend = LegendItem(horSpacing=20, box=False, verSpacing=-3, labelAlignment='right', showLines=False)
-        self.phases_legend = LegendItem(horSpacing=20, box=False, verSpacing=-3, labelAlignment='left', showLines=False)
+        self.legend = LegendItem(
+            horSpacing=20,
+            box=False,
+            verSpacing=-3,
+            labelAlignment="right",
+            showLines=False,
+        )
+        self.phases_legend = LegendItem(
+            horSpacing=20,
+            box=False,
+            verSpacing=-3,
+            labelAlignment="left",
+            showLines=False,
+        )
 
     def create_main_plot(self):
-        self.plot_item = pg.PlotDataItem(np.linspace(0, 10), np.sin(np.linspace(10, 3)),
-                                         pen=pg.mkPen(color=(255, 255, 255), width=2))
+        self.plot_item = pg.PlotDataItem(
+            np.linspace(0, 10),
+            np.sin(np.linspace(10, 3)),
+            pen=pg.mkPen(color=(255, 255, 255), width=2),
+        )
         self.pattern_plot.addItem(self.plot_item)
-        self.bkg_item = pg.PlotDataItem([], [],
-                                        pen=pg.mkPen(color=(255, 0, 0), width=2, style=QtCore.Qt.DashLine))
+        self.bkg_item = pg.PlotDataItem(
+            [], [], pen=pg.mkPen(color=(255, 0, 0), width=2, style=QtCore.Qt.DashLine)
+        )
         self.pattern_plot.addItem(self.bkg_item)
-        self.legend.addItem(self.plot_item, '')
+        self.legend.addItem(self.plot_item, "")
         self.legend.setParentItem(self.pattern_plot.vb)
         self.legend.anchor(itemPos=(1, 0), parentPos=(1, 0), offset=(-10, -10))
         self.phases_legend.setParentItem(self.pattern_plot.vb)
         self.phases_legend.anchor(itemPos=(0, 0), parentPos=(0, 0), offset=(0, -10))
 
-        self.linear_region_item = ModifiedLinearRegionItem([5, 20], pg.LinearRegionItem.Vertical, movable=False)
-        # self.linear_region_item.mouseDragEvent = empty_function
+        self.bkg_roi = ModifiedLinearRegionItem(
+            [5, 20],
+            pg.LinearRegionItem.Vertical,
+        )
+        self.map_interactive_roi = SymmetricModifiedLinearRegionItem(
+            [5, 7],
+            pg.LinearRegionItem.Vertical,
+        )
 
     @property
     def auto_range(self):
@@ -93,7 +116,9 @@ class PatternWidget(QtCore.QObject):
             self.update_graph_range()
 
     def create_pos_line(self):
-        self.pos_line = pg.InfiniteLine(pen=pg.mkPen(color=(0, 255, 0), width=1.5, style=QtCore.Qt.DashLine))
+        self.pos_line = pg.InfiniteLine(
+            pen=pg.mkPen(color=(0, 255, 0), width=1.5, style=QtCore.Qt.DashLine)
+        )
         self.pattern_plot.addItem(self.pos_line)
 
     def deactivate_pos_line(self):
@@ -120,8 +145,8 @@ class PatternWidget(QtCore.QObject):
         x_range = list(self.plot_item.dataBounds(0))
         y_range = list(self.plot_item.dataBounds(1))
 
-        for ind, overlay in enumerate(self.overlays):
-            if self.overlay_show[ind]:
+        for overlay in self.overlays:
+            if overlay in self.pattern_plot.items:
                 x_range_overlay = overlay.dataBounds(0)
                 y_range_overlay = overlay.dataBounds(1)
                 if x_range_overlay[0] < x_range[0]:
@@ -136,8 +161,7 @@ class PatternWidget(QtCore.QObject):
         if x_range[1] is not None and x_range[0] is not None:
             padding = self.view_box.suggestPadding(0)
             diff = x_range[1] - x_range[0]
-            x_range = [x_range[0] - padding * diff,
-                       x_range[1] + padding * diff]
+            x_range = [x_range[0] - padding * diff, x_range[1] + padding * diff]
 
             self.view_box.setLimits(xMin=x_range[0], xMax=x_range[1])
 
@@ -147,8 +171,7 @@ class PatternWidget(QtCore.QObject):
         if y_range[1] is not None and y_range[0] is not None:
             padding = self.view_box.suggestPadding(1)
             diff = y_range[1] - y_range[0]
-            y_range = [y_range[0] - padding * diff,
-                       y_range[1] + padding * diff]
+            y_range = [y_range[0] - padding * diff, y_range[1] + padding * diff]
 
             self.view_box.setLimits(yMin=y_range[0], yMax=y_range[1])
 
@@ -156,12 +179,13 @@ class PatternWidget(QtCore.QObject):
                 self.view_box.setRange(yRange=y_range, padding=0)
         self.emit_sig_range_changed()
 
-    def add_overlay(self, pattern, show=True):
+    def add_overlay(
+        self, pattern: Pattern, color: tuple[int, int, int], show: bool = True
+    ):
         x, y = pattern.data
-        color = calculate_color(len(self.overlays) + 1)
-        self.overlays.append(pg.PlotDataItem(x, y, pen=pg.mkPen(color=color, width=1.5)))
-        self.overlay_names.append(pattern.name)
-        self.overlay_show.append(True)
+        self.overlays.append(
+            pg.PlotDataItem(x, y, pen=pg.mkPen(color=color, width=1.5))
+        )
         if show:
             self.pattern_plot.addItem(self.overlays[-1])
             self.legend.addItem(self.overlays[-1], pattern.name)
@@ -172,67 +196,56 @@ class PatternWidget(QtCore.QObject):
         self.pattern_plot.removeItem(self.overlays[ind])
         self.legend.removeItem(self.overlays[ind])
         self.overlays.remove(self.overlays[ind])
-        self.overlay_names.remove(self.overlay_names[ind])
-        self.overlay_show.remove(self.overlay_show[ind])
-        self.update_graph_range()
 
     def hide_overlay(self, ind):
+        if not self.overlays[ind] in self.pattern_plot.items:
+            return
         self.pattern_plot.removeItem(self.overlays[ind])
         self.legend.hideItem(ind + 1)
-        self.overlay_show[ind] = False
-        self.update_graph_range()
 
     def show_overlay(self, ind):
+        if self.overlays[ind] in self.pattern_plot.items:
+            return
         self.pattern_plot.addItem(self.overlays[ind])
         self.legend.showItem(ind + 1)
-        self.overlay_show[ind] = True
+
+    def update_overlay(self, ind: int, overlay: Overlay):
+        self.set_overlay_data(ind, overlay.x, overlay.y)
+        self.set_overlay_color(ind, overlay.color)
+        self.rename_overlay(ind, overlay.name)
+        if overlay.visible:
+            self.show_overlay(ind)
+        else:
+            self.hide_overlay(ind)
         self.update_graph_range()
 
-    def update_overlay(self, pattern, ind):
-        x, y = pattern.data
+    def set_overlay_data(self, ind, x, y):
         self.overlays[ind].setData(x, y)
-        self.update_graph_range()
 
-    def set_overlay_color(self, ind, color):
+    def set_overlay_color(self, ind: int, color):
         self.overlays[ind].setPen(pg.mkPen(color=color, width=1.5))
         self.legend.setItemColor(ind + 1, color)
 
-    def rename_overlay(self, ind, name):
+    def rename_overlay(self, ind: int, name: str):
         self.legend.renameItem(ind + 1, name)
 
-    def move_overlay_up(self, ind):
-        new_ind = ind - 1
-        self.overlays.insert(new_ind, self.overlays.pop(ind))
-        self.overlay_names.insert(new_ind, self.overlay_names.pop(ind))
-        self.overlay_show.insert(new_ind, self.overlay_show.pop(ind))
-
-        color = self.legend.legendItems[ind + 1][1].opts['color']
-        label = self.legend.legendItems[ind + 1][1].text
-        self.legend.legendItems[ind + 1][1].setAttr('color', self.legend.legendItems[new_ind + 1][1].opts['color'])
-        self.legend.legendItems[ind + 1][1].setText(self.legend.legendItems[new_ind + 1][1].text)
-        self.legend.legendItems[new_ind + 1][1].setAttr('color', color)
-        self.legend.legendItems[new_ind + 1][1].setText(label)
-
-    def move_overlay_down(self, cur_ind):
-        self.overlays.insert(cur_ind + 1, self.overlays.pop(cur_ind))
-        self.overlay_names.insert(cur_ind + 1, self.overlay_names.pop(cur_ind))
-        self.overlay_show.insert(cur_ind + 1, self.overlay_show.pop(cur_ind))
-
-        color = self.legend.legendItems[cur_ind + 1][1].opts['color']
-        label = self.legend.legendItems[cur_ind + 1][1].text
-        self.legend.legendItems[cur_ind + 1][1].setAttr('color', self.legend.legendItems[cur_ind + 2][1].opts['color'])
-        self.legend.legendItems[cur_ind + 1][1].setText(self.legend.legendItems[cur_ind + 2][1].text)
-        self.legend.legendItems[cur_ind + 2][1].setAttr('color', color)
-        self.legend.legendItems[cur_ind + 2][1].setText(label)
-
     def set_antialias(self, value):
         for overlay in self.overlays:
-            overlay.opts['antialias'] = value
+            overlay.opts["antialias"] = value
             overlay.updateItems()
 
     def add_phase(self, name, positions, intensities, baseline, color):
         self.phases.append(
-            PhasePlot(self.pattern_plot, self.phases_legend, positions, intensities, color, name, baseline))
+            PhasePlot(
+                self.pattern_plot,
+                self.phases_legend,
+                positions,
+                intensities,
+                color,
+                name,
+                baseline,
+            )
+        )
 
     def set_phase_color(self, ind, color):
         self.phases[ind].set_color(color)
@@ -270,21 +283,38 @@ class PatternWidget(QtCore.QObject):
         if len(self.phases_vlines) > 0:
             self.phases_vlines[0].set_data(positions, name)
         else:
-            self.phases_vlines.append(PhaseLinesPlot(self.pattern_plot, positions,
-                                                     pen=pg.mkPen(color=(200, 50, 50), style=QtCore.Qt.SolidLine)))
+            self.phases_vlines.append(
+                PhaseLinesPlot(
+                    self.pattern_plot,
+                    positions,
+                    pen=pg.mkPen(color=(200, 50, 50), style=QtCore.Qt.SolidLine),
+                )
+            )
 
-    def show_linear_region(self):
-        self.pattern_plot.addItem(self.linear_region_item)
+    # bkg_linear region functions
+    def show_bkg_roi(self):
+        self.pattern_plot.addItem(self.bkg_roi)
 
-    def set_linear_region(self, x_min, x_max):
-        self.linear_region_item.setRegion((x_min, x_max))
+    def set_bkg_roi(self, x_min, x_max):
+        self.bkg_roi.setRegion((x_min, x_max))
 
-    def get_linear_region(self):
-        return self.linear_region_item.getRegion()
+    def get_bkg_roi(self):
+        return self.bkg_roi.getRegion()
 
-    def hide_linear_region(self):
-        self.pattern_plot.removeItem(self.linear_region_item)
+    def hide_bkg_roi(self):
+        self.pattern_plot.removeItem(self.bkg_roi)
 
+    # map interactive linear region functions
+    def show_map_interactive_roi(self):
+        self.pattern_plot.addItem(self.map_interactive_roi)
+
+    def set_map_interactive_roi(self, x_min, x_max):
+        self.map_interactive_roi.setRegion((x_min, x_max))
+
+    def hide_map_interactive_roi(self):
+        self.pattern_plot.removeItem(self.map_interactive_roi)
+
+    # saving the plot
     def save_png(self, filename):
         exporter = ImageExporter(self.pattern_plot)
         exporter.export(filename)
@@ -292,28 +322,30 @@ class PatternWidget(QtCore.QObject):
     def save_svg(self, filename):
         self._invert_color()
         previous_label = None
-        if self.pattern_plot.getAxis('bottom').labelText == u'2θ':
-            previous_label = (u'2θ', '°')
-            self.pattern_plot.setLabel('bottom', '2th_deg', '')
+        if self.pattern_plot.getAxis("bottom").labelText == "2θ":
+            previous_label = ("2θ", "°")
+            self.pattern_plot.setLabel("bottom", "2th_deg", "")
         exporter = SVGExporter(self.pattern_plot)
         exporter.export(filename)
         self._norm_color()
         if previous_label is not None:
-            self.pattern_plot.setLabel('bottom', previous_label[0], previous_label[1])
+            self.pattern_plot.setLabel("bottom", previous_label[0], previous_label[1])
 
     def _invert_color(self):
-        self.pattern_plot.getAxis('bottom').setPen('k')
-        self.pattern_plot.getAxis('left').setPen('k')
-        self.plot_item.setPen('k')
-        self.legend.legendItems[0][1].setAttr('color', '000')
-        self.legend.legendItems[0][1].setText(self.legend.legendItems[0][1].text)
+        self.pattern_plot.getAxis("bottom").setPen("k")
+        self.pattern_plot.getAxis("left").setPen("k")
+        self.plot_item.setPen("k")
+        self.legend.legendItems[0][1].setText(
+            self.legend.legendItems[0][1].text, color="#000"
+        )
 
     def _norm_color(self):
-        self.pattern_plot.getAxis('bottom').setPen('w')
-        self.pattern_plot.getAxis('left').setPen('w')
-        self.plot_item.setPen('w')
-        self.legend.legendItems[0][1].setAttr('color', 'FFF')
-        self.legend.legendItems[0][1].setText(self.legend.legendItems[0][1].text)
+        self.pattern_plot.getAxis("bottom").setPen("w")
+        self.pattern_plot.getAxis("left").setPen("w")
+        self.plot_item.setPen("w")
+        self.legend.legendItems[0][1].setText(
+            self.legend.legendItems[0][1].text, color="#FFF"
+        )
 
     def mouseMoved(self, pos):
         pos = self.plot_item.mapFromScene(pos)
@@ -338,9 +370,10 @@ class PatternWidget(QtCore.QObject):
         self.last_view_range = np.array(self.view_box.viewRange())
 
     def myMouseClickEvent(self, ev):
-        if ev.button() == QtCore.Qt.RightButton or \
-                (ev.button() == QtCore.Qt.LeftButton and
-                 ev.modifiers() & QtCore.Qt.ControlModifier):
+        if ev.button() == QtCore.Qt.RightButton or (
+            ev.button() == QtCore.Qt.LeftButton
+            and ev.modifiers() & QtCore.Qt.ControlModifier
+        ):
             view_range = np.array(self.view_box.viewRange()) * 2
             curve_data = self.plot_item.getData()
             x_range = np.max(curve_data[0]) - np.min(curve_data[0])
@@ -358,8 +391,10 @@ class PatternWidget(QtCore.QObject):
             self.mouse_left_clicked.emit(x, y)
 
     def myMouseDoubleClickEvent(self, ev):
-        if (ev.button() == QtCore.Qt.RightButton) or (ev.button() == QtCore.Qt.LeftButton and
-                                                      ev.modifiers() & QtCore.Qt.ControlModifier):
+        if (ev.button() == QtCore.Qt.RightButton) or (
+            ev.button() == QtCore.Qt.LeftButton
+            and ev.modifiers() & QtCore.Qt.ControlModifier
+        ):
             self.auto_range = True
             self.emit_sig_range_changed()
 
@@ -371,9 +406,10 @@ class PatternWidget(QtCore.QObject):
         dif = pos - lastPos
         dif *= -1
 
-        if ev.button() == QtCore.Qt.RightButton or \
-                (ev.button() == QtCore.Qt.LeftButton and
-                 ev.modifiers() & QtCore.Qt.ControlModifier):
+        if ev.button() == QtCore.Qt.RightButton or (
+            ev.button() == QtCore.Qt.LeftButton
+            and ev.modifiers() & QtCore.Qt.ControlModifier
+        ):
             # determine the amount of translation
             tr = dif
             tr = self.view_box.mapToView(tr) - self.view_box.mapToView(pg.Point(0, 0))
@@ -386,14 +422,20 @@ class PatternWidget(QtCore.QObject):
                 self.range_changed_timer.stop()
                 self.emit_sig_range_changed()
         else:
-            if ev.isFinish():  # This is the final move in the drag; change the view scale now
+            if (
+                ev.isFinish()
+            ):  # This is the final move in the drag; change the view scale now
                 self.auto_range = False
                 self.view_box.rbScaleBox.hide()
-                ax = QtCore.QRectF(pg.Point(ev.buttonDownPos(ev.button())), pg.Point(pos))
+                ax = QtCore.QRectF(
+                    pg.Point(ev.buttonDownPos(ev.button())), pg.Point(pos)
+                )
                 ax = self.view_box.childGroup.mapRectFromParent(ax)
                 self.view_box.showAxRect(ax)
                 self.view_box.axHistoryPointer += 1
-                self.view_box.axHistory = self.view_box.axHistory[:self.view_box.axHistoryPointer] + [ax]
+                self.view_box.axHistory = self.view_box.axHistory[
+                    : self.view_box.axHistoryPointer
+                ] + [ax]
                 self.emit_sig_range_changed()
             else:
                 # update shape of scale box
@@ -402,7 +444,9 @@ class PatternWidget(QtCore.QObject):
     def emit_sig_range_changed(self):
         new_view_range = np.array(self.view_box.viewRange())
         if not np.array_equal(self.last_view_range, new_view_range):
-            self.view_box.sigRangeChangedManually.emit(self.view_box.state['mouseEnabled'])
+            self.view_box.sigRangeChangedManually.emit(
+                self.view_box.state["mouseEnabled"]
+            )
             self.last_view_range = new_view_range
 
     def myWheelEvent(self, ev, axis=None, *args):
@@ -417,20 +461,27 @@ class PatternWidget(QtCore.QObject):
             if self.auto_range is not True:
                 view_range = np.array(self.view_box.viewRange())
                 curve_data = self.plot_item.getData()
-                x_range = np.max(curve_data[0]) - np.min(curve_data[0])
-                y_range = np.max(curve_data[1]) - np.min(curve_data[1])
-                if (view_range[0][1] - view_range[0][0]) >= x_range and \
-                        (view_range[1][1] - view_range[1][0]) >= y_range:
-                    self.auto_range = True
-                else:
-                    self.auto_range = False
+                self.auto_range = False
+                if len(curve_data[0]) > 2:
+                    x_range = np.max(curve_data[0]) - np.min(curve_data[0])
+                    y_range = np.max(curve_data[1]) - np.min(curve_data[1])
+                    if (view_range[0][1] - view_range[0][0]) >= x_range and (
+                        view_range[1][1] - view_range[1][0]
+                    ) >= y_range:
+                        self.auto_range = True
+                if not self.auto_range:
                     pg.ViewBox.wheelEvent(self.view_box, ev)
             self.emit_sig_range_changed()
 
 
 class PhaseLinesPlot(object):
-    def __init__(self, plot_item, positions=None, name='Dummy',
-                 pen=pg.mkPen(color=(120, 120, 120), style=QtCore.Qt.DashLine)):
+    def __init__(
+        self,
+        plot_item,
+        positions=None,
+        name="Dummy",
+        pen=pg.mkPen(color=(120, 120, 120), style=QtCore.Qt.DashLine),
+    ):
         self.plot_item = plot_item
         self.peak_positions = []
         self.line_items = []
@@ -457,7 +508,16 @@ class PhaseLinesPlot(object):
 class PhasePlot(object):
     num_phases = 0
 
-    def __init__(self, plot_item, legend_item, positions, intensities, color, name=None, baseline=0):
+    def __init__(
+        self,
+        plot_item,
+        legend_item,
+        positions,
+        intensities,
+        color,
+        name=None,
+        baseline=0,
+    ):
         self.plot_item = plot_item
         self.legend_item = legend_item
         self.visible = True
@@ -468,7 +528,7 @@ class PhasePlot(object):
         self.color = color
         self.pen = pg.mkPen(color=self.color, width=0.9, style=QtCore.Qt.SolidLine)
         self.ref_legend_line = pg.PlotDataItem(pen=self.pen)
-        self.name = ''
+        self.name = ""
         PhasePlot.num_phases += 1
         self.create_items(positions, intensities, name, baseline)
 
@@ -477,10 +537,14 @@ class PhasePlot(object):
         self.line_items = []
 
         for ind, position in enumerate(positions):
-            self.line_items.append(pg.PlotDataItem(x=[position, position],
-                                                   y=[baseline, intensities[ind]],
-                                                   pen=self.pen,
-                                                   antialias=False))
+            self.line_items.append(
+                pg.PlotDataItem(
+                    x=[position, position],
+                    y=[baseline, intensities[ind]],
+                    pen=self.pen,
+                    antialias=False,
+                )
+            )
             self.line_visible.append(True)
             self.plot_item.addItem(self.line_items[ind])
 
@@ -492,9 +556,9 @@ class PhasePlot(object):
                 pass
 
     def add_line(self):
-        self.line_items.append(pg.PlotDataItem(x=[0, 0],
-                                               y=[0, 0],
-                                               pen=self.pen, antialias=False))
+        self.line_items.append(
+            pg.PlotDataItem(x=[0, 0], y=[0, 0], pen=self.pen, antialias=False)
+        )
         self.line_visible.append(True)
         self.plot_item.blockSignals(True)
         self.plot_item.addItem(self.line_items[-1])
@@ -512,11 +576,12 @@ class PhasePlot(object):
     def update_intensities(self, positions, intensities, baseline=0):
         if self.visible:
             for ind, intensity in enumerate(intensities):
-                self.line_items[ind].setData(y=[baseline, intensity],
-                                             x=[positions[ind], positions[ind]])
+                self.line_items[ind].setData(
+                    y=[baseline, intensity], x=[positions[ind], positions[ind]]
+                )
 
     def update_visibilities(self, pattern_range):
-        if self.visible:
+        if self.visible and pattern_range[0] is not None:
             for ind, line_item in enumerate(self.line_items):
                 data = line_item.getData()
                 position = data[0][0]
@@ -553,7 +618,7 @@ class PhasePlot(object):
         try:
             self.legend_item.removeItem(self.ref_legend_line)
         except IndexError:
-            print('this phase had now lines in the appropriate region')
+            print("this phase had now lines in the appropriate region")
         for ind, item in enumerate(self.line_items):
             if self.line_visible[ind]:
                 self.plot_item.removeItem(item)
@@ -561,10 +626,25 @@ class PhasePlot(object):
 
 class ModifiedLinearRegionItem(pg.LinearRegionItem):
     def __init__(self, *args, **kwargs):
-        super(ModifiedLinearRegionItem, self).__init__()
+        super(ModifiedLinearRegionItem, self).__init__(*args, **kwargs)
 
     def mouseDragEvent(self, ev):
         return
 
     def hoverEvent(self, ev):
         return
+
+
+class SymmetricModifiedLinearRegionItem(pg.LinearRegionItem):
+    def __init__(self, *args, **kwargs):
+        super(SymmetricModifiedLinearRegionItem, self).__init__(*args, **kwargs)
+        self.center = (
+            self.lines[1].getXPos() - self.lines[0].getXPos()
+        ) / 2 + self.lines[0].getXPos()
+
+    def setCenter(self, center):
+        previous_width = self.lines[1].value() - self.lines[0].value()
+        new_left = center - previous_width / 2
+        new_right = center + previous_width / 2
+        self.center = center
+        self.setRegion([new_left, new_right])
